""" Test access to views
"""
from unittest.mock import patch, MagicMock

from django.contrib.auth.models import AnonymousUser
from django.contrib.messages.middleware import MessageMiddleware
from django.contrib.messages.storage.fallback import FallbackStorage
from django.contrib.sessions.middleware import SessionMiddleware
from django.core.files.uploadedfile import SimpleUploadedFile
from django.http import QueryDict
from django.test import RequestFactory
from tests.components.data.fixtures.fixtures import (
    AccessControlBlobWithMetadataFixture,
)
from tests.views.fixtures import AccessControlDataFixture

from core_main_app.access_control.exceptions import AccessControlError
from core_main_app.commons.exceptions import ModelError
from core_main_app.components.blob import api as blob_api
from core_main_app.components.data import api as data_api
from core_main_app.settings import MAX_DOCUMENT_EDITING_SIZE
from core_main_app.utils.integration_tests.integration_base_test_case import (
    IntegrationBaseTestCase,
)
from core_main_app.utils.tests_tools.MockUser import create_mock_user
from core_main_app.views.common.views import (
    ViewData,
    EditWorkspaceRights,
    TemplateXSLRenderingView,
    DataContentEditor,
    XSDEditor,
    AbstractEditorView,
    ViewBlob,
    ManageBlobMetadata,
)
from core_main_app.views.user.ajax import (
    AssignView,
    change_data_display,
    LoadFormChangeWorkspace,
    load_add_user_form,
    add_user_right_to_workspace,
    switch_right,
    remove_user_or_group_rights,
    load_add_group_form,
    add_group_right_to_workspace,
    UploadFile,
    RemoveMetadataFromBlob,
    AddMetadataToBlob,
    LoadBlobMetadataForm,
)
from core_main_app.views.user.views import manage_template_versions
from tests.test_settings import LOGIN_URL


class TestViewData(IntegrationBaseTestCase):
    """TestViewData"""

    def setUp(self):
        """setUp

        Returns:

        """
        self.factory = RequestFactory()
        self.user1 = create_mock_user(user_id="1")
        self.anonymous = AnonymousUser()
        self.fixture = AccessControlDataFixture()
        self.fixture.insert_data()

    def test_a_user_can_access_a_data_if_owner(self):
        """test_a_user_can_access_a_data_if_owner

        Returns:

        """
        request = self.factory.get("core_main_app_data_detail")
        request.user = self.user1
        request.GET = {"id": str(self.fixture.data_1.id)}
        response = ViewData.as_view()(request)
        self.assertTrue(self.fixture.data_1.title in response.content.decode())

    def test_an_anonymous_user_can_not_access_a_data_that_is_not_in_a_workspace(
        self,
    ):
        """test_an_anonymous_user_can_not_access_a_data_that_is_not_in_a_workspace

        Returns:

        """
        request = self.factory.get("core_main_app_data_detail")
        request.user = self.anonymous
        request.GET = {"id": str(self.fixture.data_1.id)}
        response = ViewData.as_view()(request)
        self.assertTrue(
            self.fixture.data_1.title not in response.content.decode()
        )
        self.assertTrue("Error 403" in response.content.decode())

    def test_an_anonymous_user_can_not_access_a_data_that_is_in_a_private_workspace(
        self,
    ):
        """test_an_anonymous_user_can_not_access_a_data_that_is_in_a_private_workspace

        Returns:

        """
        request = self.factory.get("core_main_app_data_detail")
        request.user = self.anonymous
        request.GET = {"id": str(self.fixture.data_1.id)}
        response = ViewData.as_view()(request)
        self.assertTrue(
            self.fixture.data_1.title not in response.content.decode()
        )
        self.assertTrue("Error" in response.content.decode())

    def test_an_anonymous_user_can_not_access_a_data_that_is_in_a_public_workspace_and_access_setting_is_false(
        self,
    ):
        """test_an_anonymous_user_can_not_access_a_data_that_is_in_a_public_workspace_and_access_setting_is_false

        Returns:

        """
        request = self.factory.get("core_main_app_data_detail")
        request.user = self.anonymous
        request.GET = {"id": str(self.fixture.data_public_workspace.id)}
        response = ViewData.as_view()(request)
        self.assertTrue(
            self.fixture.data_public_workspace.title
            not in response.content.decode()
        )
        self.assertTrue("Error" in response.content.decode())


class TestManageTemplateVersions(IntegrationBaseTestCase):
    """TestManageTemplateVersions"""

    def setUp(self):
        """setUp

        Returns:

        """
        self.factory = RequestFactory()
        self.user1 = create_mock_user(user_id="1")
        self.anonymous = AnonymousUser()
        self.fixture = AccessControlDataFixture()
        self.fixture.insert_data()

    def test_an_anonymous_user_can_not_manage_template_versions(self):
        """test_an_anonymous_user_can_not_manage_template_versions

        Returns:

        """
        request = self.factory.get("core_main_app_manage_template_versions")
        request.user = self.anonymous
        response = manage_template_versions(request)
        self.assertEqual(response.status_code, 302)
        self.assertTrue(response.url.startswith(LOGIN_URL))


class TestTemplateXSLRenderingView(IntegrationBaseTestCase):
    """TestTemplateXSLRenderingView"""

    def setUp(self):
        """setUp

        Returns:

        """
        self.factory = RequestFactory()
        self.user1 = create_mock_user(user_id="1")
        self.anonymous = AnonymousUser()
        self.fixture = AccessControlDataFixture()
        self.fixture.insert_data()

    def test_an_anonymous_user_can_not_manage_template_versions(self):
        """test_an_anonymous_user_can_not_manage_template_versions

        Returns:

        """
        request = self.factory.get("core_main_app_template_xslt")
        request.user = self.anonymous
        response = TemplateXSLRenderingView.as_view()(request)
        self.assertEqual(response.status_code, 302)
        self.assertTrue(response.url.startswith(LOGIN_URL))


class TestAssignDataView(IntegrationBaseTestCase):
    """TestAssignDataView"""

    def setUp(self):
        """setUp

        Returns:

        """
        self.factory = RequestFactory()
        self.user1 = create_mock_user(user_id="1")
        self.anonymous = AnonymousUser()
        self.fixture = AccessControlDataFixture()
        self.fixture.insert_data()

    def test_a_user_can_assign_a_data_if_owner(self):
        """test_a_user_can_assign_a_data_if_owner

        Returns:

        """
        data = {
            "document_id[]": [str(self.fixture.data_1.id)],
            "workspace_id[]": [str(self.fixture.workspace_2.id)],
        }
        request = self.factory.post("core_main_assign_data_workspace", data)
        request.user = self.user1
        response = AssignView.as_view(api=data_api)(request)
        self.assertEqual(response.status_code, 200)

    def test_an_anonymous_user_can_not_assign_a_user_data(self):
        """test_an_anonymous_user_can_not_assign_a_user_data

        Returns:

        """
        data = {
            "document_id[]": [str(self.fixture.data_1.id)],
            "workspace_id[]": [str(self.fixture.workspace_2.id)],
        }
        request = self.factory.post("core_main_assign_data_workspace", data)
        request.user = self.anonymous
        response = AssignView.as_view(api=data_api)(request)
        self.assertEqual(response.status_code, 302)
        self.assertTrue(response.url.startswith(LOGIN_URL))

    def test_an_anonymous_user_can_not_assign_a_workspace_data(self):
        """test_an_anonymous_user_can_not_assign_a_workspace_data

        Returns:

        """
        data = {
            "document_id[]": [str(self.fixture.data_workspace_1.id)],
            "workspace_id[]": [str(self.fixture.workspace_2.id)],
        }
        request = self.factory.post("core_main_assign_data_workspace", data)
        request.user = self.anonymous
        response = AssignView.as_view(api=data_api)(request)
        self.assertEqual(response.status_code, 302)
        self.assertTrue(response.url.startswith(LOGIN_URL))

    def test_an_anonymous_user_can_not_assign_a_public_workspace_data(self):
        """test_an_anonymous_user_can_not_assign_a_public_workspace_data

        Returns:

        """
        data = {
            "document_id[]": [str(self.fixture.data_public_workspace.id)],
            "workspace_id[]": [str(self.fixture.workspace_2.id)],
        }
        request = self.factory.post("core_main_assign_data_workspace", data)
        request.user = self.anonymous
        response = AssignView.as_view(api=data_api)(request)
        self.assertEqual(response.status_code, 302)
        self.assertTrue(response.url.startswith(LOGIN_URL))


class TestAssignBlobView(IntegrationBaseTestCase):
    """TestAssignBlobView"""

    def setUp(self):
        """setUp

        Returns:

        """
        self.factory = RequestFactory()
        self.user1 = create_mock_user(user_id="1")
        self.anonymous = AnonymousUser()
        self.fixture = AccessControlDataFixture()
        self.fixture.insert_data()

    def test_a_user_can_assign_a_blob_if_owner(self):
        """test_a_user_can_assign_a_blob_if_owner

        Returns:

        """
        data = {
            "document_id[]": [str(self.fixture.blob_1.id)],
            "workspace_id[]": [str(self.fixture.workspace_2.id)],
        }
        request = self.factory.post("core_main_assign_blob_workspace", data)
        request.user = self.user1
        response = AssignView.as_view(api=blob_api)(request)
        self.assertEqual(response.status_code, 200)

    def test_an_anonymous_user_can_not_assign_a_user_blob(self):
        """test_an_anonymous_user_can_not_assign_a_user_blob

        Returns:

        """
        data = {
            "document_id[]": [str(self.fixture.blob_1.id)],
            "workspace_id[]": [str(self.fixture.workspace_2.id)],
        }
        request = self.factory.post("core_main_assign_blob_workspace", data)
        request.user = self.anonymous
        response = AssignView.as_view(api=blob_api)(request)
        self.assertEqual(response.status_code, 302)
        self.assertTrue(response.url.startswith(LOGIN_URL))

    def test_an_anonymous_user_can_not_assign_a_workspace_blob(self):
        """test_an_anonymous_user_can_not_assign_a_workspace_blob

        Returns:

        """
        data = {
            "document_id[]": [str(self.fixture.blob_workspace_1.id)],
            "workspace_id[]": [str(self.fixture.workspace_2.id)],
        }
        request = self.factory.post("core_main_assign_blob_workspace", data)
        request.user = self.anonymous
        response = AssignView.as_view(api=blob_api)(request)
        self.assertEqual(response.status_code, 302)
        self.assertTrue(response.url.startswith(LOGIN_URL))

    def test_an_anonymous_user_can_not_assign_a_public_workspace_blob(self):
        """test_an_anonymous_user_can_not_assign_a_public_workspace_blob

        Returns:

        """
        data = {
            "document_id[]": [str(self.fixture.blob_public_workspace.id)],
            "workspace_id[]": [str(self.fixture.workspace_2.id)],
        }
        request = self.factory.post("core_main_assign_blob_workspace", data)
        request.user = self.anonymous
        response = AssignView.as_view(api=blob_api)(request)
        self.assertEqual(response.status_code, 302)
        self.assertTrue(response.url.startswith(LOGIN_URL))


class TestChangeDataDisplayView(IntegrationBaseTestCase):
    """TestChangeDataDisplayView"""

    def setUp(self):
        """setUp

        Returns:

        """
        self.factory = RequestFactory()
        self.user1 = create_mock_user(user_id="1")
        self.anonymous = AnonymousUser()
        self.fixture = AccessControlDataFixture()
        self.fixture.insert_data()

    def test_a_user_can_change_data_display_if_owner(self):
        """test_a_user_can_change_data_display_if_owner

        Returns:

        """
        data = {"xslt_id": "id", "data_id": str(self.fixture.data_1.id)}
        request = self.factory.post("core_main_add_change_data_display", data)
        request.user = self.user1
        response = change_data_display(request)
        self.assertEqual(response.status_code, 200)

    def test_change_data_display_returns_http_200(self):
        """test_change_data_display_returns_http_200

        Returns:

        """
        data = {
            "xslt_id": "1",
            "template_id": str(self.fixture.data_1.template.id),
            "content": "<tag></tag>",
        }
        request = self.factory.post("core_main_add_change_data_display", data)
        request.user = self.user1
        response = change_data_display(request)
        self.assertEqual(response.status_code, 200)

    def test_change_data_display_returns_error_when_template_id_is_missing(
        self,
    ):
        """test_change_data_display_returns_error_when_template_id_is_missing

        Returns:

        """
        data = {
            "xslt_id": "1",
            "content": "<tag></tag>",
        }
        request = self.factory.post("core_main_add_change_data_display", data)
        request.user = self.user1
        response = change_data_display(request)
        self.assertEqual(response.status_code, 400)

    def test_an_anonymous_user_can_not_assign_a_user_data(self):
        """test_an_anonymous_user_can_not_assign_a_user_data

        Returns:

        """
        data = {"xslt_id": "id", "data_id": str(self.fixture.data_1.id)}
        request = self.factory.post("core_main_add_change_data_display", data)
        request.user = self.anonymous
        response = change_data_display(request)
        self.assertEqual(response.status_code, 403)

    def test_an_anonymous_user_can_not_assign_a_workspace_data(self):
        """test_an_anonymous_user_can_not_assign_a_workspace_data

        Returns:

        """
        data = {
            "xslt_id": "id",
            "data_id": str(self.fixture.data_workspace_1.id),
        }
        request = self.factory.post("core_main_add_change_data_display", data)
        request.user = self.anonymous
        response = change_data_display(request)
        self.assertEqual(response.status_code, 403)

    def test_an_anonymous_user_can_not_assign_a_public_workspace_data(self):
        """test_an_anonymous_user_can_not_assign_a_public_workspace_data

        Returns:

        """
        data = {
            "xslt_id": "id",
            "data_id": str(self.fixture.data_public_workspace.id),
        }
        request = self.factory.post("core_main_add_change_data_display", data)
        request.user = self.anonymous
        response = change_data_display(request)
        self.assertEqual(response.status_code, 403)


class TestEditRights(IntegrationBaseTestCase):
    """TestEditRights"""

    def setUp(self):
        """setUp

        Returns:

        """
        self.factory = RequestFactory()
        self.user1 = create_mock_user(user_id="1")
        self.anonymous = AnonymousUser()
        self.fixture = AccessControlDataFixture()
        self.fixture.insert_data()

    def test_an_anonymous_user_can_not_edit_rights_workspace(self):
        """test_an_anonymous_user_can_not_edit_rights_workspace

        Returns:

        """
        request = self.factory.post("core_main_edit_rights_workspace")
        request.user = self.anonymous
        response = EditWorkspaceRights.as_view()(request)
        self.assertEqual(response.status_code, 302)
        self.assertTrue(response.url.startswith(LOGIN_URL))


class TestLoadFormChangeWorkspace(IntegrationBaseTestCase):
    """TestLoadFormChangeWorkspace"""

    def setUp(self):
        """setUp

        Returns:

        """
        self.factory = RequestFactory()
        self.user1 = create_mock_user(user_id="1")
        self.anonymous = AnonymousUser()
        self.fixture = AccessControlDataFixture()
        self.fixture.insert_data()

    def test_an_anonymous_user_can_not_load_form_change_workspace(self):
        """test_an_anonymous_user_can_not_load_form_change_workspace

        Returns:

        """
        data = {"administration": "True"}
        request = self.factory.post("core_main_change_workspace", data)
        request.user = self.anonymous
        response = LoadFormChangeWorkspace.as_view()(request)
        self.assertEqual(response.status_code, 302)
        self.assertTrue(response.url.startswith(LOGIN_URL))


class TestLoadAddUserForm(IntegrationBaseTestCase):
    """TestLoadAddUserForm"""

    def setUp(self):
        """setUp

        Returns:

        """
        self.factory = RequestFactory()
        self.user1 = create_mock_user(user_id="1")
        self.anonymous = AnonymousUser()
        self.fixture = AccessControlDataFixture()
        self.fixture.insert_data()

    def test_an_anonymous_user_can_not_load_add_user_form(self):
        """test_an_anonymous_user_can_not_load_add_user_form

        Returns:

        """
        request = self.factory.post("core_main_edit_rights_users_form")
        request.user = self.anonymous
        response = load_add_user_form(request)
        self.assertEqual(response.status_code, 302)
        self.assertTrue(response.url.startswith(LOGIN_URL))


class TestAddUserToWorkspace(IntegrationBaseTestCase):
    """TestAddUserToWorkspace"""

    def setUp(self):
        """setUp

        Returns:

        """
        self.factory = RequestFactory()
        self.user1 = create_mock_user(user_id="1")
        self.anonymous = AnonymousUser()
        self.fixture = AccessControlDataFixture()
        self.fixture.insert_data()

    def test_an_anonymous_user_can_not_add_user_to_workspace(self):
        """test_an_anonymous_user_can_not_add_user_to_workspace

        Returns:

        """
        request = self.factory.post("core_main_add_user_right_to_workspace")
        request.user = self.anonymous
        response = add_user_right_to_workspace(request)
        self.assertEqual(response.status_code, 302)
        self.assertTrue(response.url.startswith(LOGIN_URL))


class TestSwitchRight(IntegrationBaseTestCase):
    """TestSwitchRight"""

    def setUp(self):
        """setUp

        Returns:

        """
        self.factory = RequestFactory()
        self.user1 = create_mock_user(user_id="1")
        self.anonymous = AnonymousUser()
        self.fixture = AccessControlDataFixture()
        self.fixture.insert_data()

    def test_an_anonymous_user_can_not_switch_right(self):
        """test_an_anonymous_user_can_not_switch_right

        Returns:

        """
        request = self.factory.post("core_main_switch_right")
        request.user = self.anonymous
        response = switch_right(request)
        self.assertEqual(response.status_code, 302)
        self.assertTrue(response.url.startswith(LOGIN_URL))


class TestRemoveRights(IntegrationBaseTestCase):
    """TestRemoveRights"""

    def setUp(self):
        """setUp

        Returns:

        """
        self.factory = RequestFactory()
        self.user1 = create_mock_user(user_id="1")
        self.anonymous = AnonymousUser()
        self.fixture = AccessControlDataFixture()
        self.fixture.insert_data()

    def test_an_anonymous_user_can_not_remove_rights(self):
        """test_an_anonymous_user_can_not_remove_rights

        Returns:

        """
        request = self.factory.post("core_main_remove_rights")
        request.user = self.anonymous
        response = remove_user_or_group_rights(request)
        self.assertEqual(response.status_code, 302)
        self.assertTrue(response.url.startswith(LOGIN_URL))


class TestAddGroupForm(IntegrationBaseTestCase):
    """TestAddGroupForm"""

    def setUp(self):
        """setUp

        Returns:

        """
        self.factory = RequestFactory()
        self.user1 = create_mock_user(user_id="1")
        self.anonymous = AnonymousUser()
        self.fixture = AccessControlDataFixture()
        self.fixture.insert_data()

    def test_an_anonymous_user_can_not_add_group_form(self):
        """test_an_anonymous_user_can_not_add_group_form

        Returns:

        """
        request = self.factory.post("core_main_edit_rights_groups_form")
        request.user = self.anonymous
        response = load_add_group_form(request)
        self.assertEqual(response.status_code, 302)
        self.assertTrue(response.url.startswith(LOGIN_URL))


class TestAddGroupRightToWorkspace(IntegrationBaseTestCase):
    """TestAddGroupRightToWorkspace"""

    def setUp(self):
        """setUp

        Returns:

        """
        self.factory = RequestFactory()
        self.user1 = create_mock_user(user_id="1")
        self.anonymous = AnonymousUser()
        self.fixture = AccessControlDataFixture()
        self.fixture.insert_data()

    def test_an_anonymous_user_can_not_add_group_right_to_workspace(self):
        """test_an_anonymous_user_can_not_add_group_right_to_workspace

        Returns:

        """
        request = self.factory.post("core_main_add_group_right_to_workspace")
        request.user = self.anonymous
        response = add_group_right_to_workspace(request)
        self.assertEqual(response.status_code, 302)
        self.assertTrue(response.url.startswith(LOGIN_URL))


class TestAbstractEditorView(IntegrationBaseTestCase):
    """Test Get XML Text Editor View"""

    def setUp(self):

        """setUp

        Returns:

        """
        self.fixture = AccessControlDataFixture()
        self.fixture.insert_data()

    def test_format_raises_not_implemented_error(self):
        """test_format_raises_not_implemented_error

        Returns:

        """
        # Assert
        with self.assertRaises(NotImplementedError):
            AbstractEditorView.format(self)

    def test_validate_raises_not_implemented_error(self):
        """test_validate_raises_not_implemented_error

        Returns:

        """
        # Assert
        with self.assertRaises(NotImplementedError):
            AbstractEditorView.validate(self)

    def test_save_raises_not_implemented_error(self):
        """test_save_raises_not_implemented_error

        Returns:

        """
        # Assert
        with self.assertRaises(NotImplementedError):
            AbstractEditorView.save(self)


class TestDataContentEditorView(IntegrationBaseTestCase):
    """Test Post Data Content Editor View"""

    def setUp(self):
        """setUp

        Returns:

        """
        self.factory = RequestFactory()
        self.user1 = create_mock_user(user_id="1")
        self.anonymous = AnonymousUser()
        self.fixture = AccessControlDataFixture()
        self.fixture.insert_data()

    def test_anonymous_user_can_not_access_to_data(self):
        """test_anonymous_user_can_not_access_to_data

        Returns:

        """
        request = self.factory.get("core_main_app_xml_text_editor_view")
        request.user = self.anonymous
        request.GET = {"id": str(self.fixture.data_1.id)}
        response = DataContentEditor.as_view()(request)
        self.assertTrue(
            self.fixture.data_1.title not in response.content.decode()
        )
        self.assertTrue("Error 403" in response.content.decode())

    def test_user_can_not_access_to_data_if_not_found(self):
        """test_user_can_not_access_a_data_if_not_found

        Returns:

        """
        request = self.factory.get("core_main_app_xml_text_editor_view")
        request.GET = {"id": "-1"}
        request.user = self.user1
        response = DataContentEditor.as_view()(request)
        self.assertTrue("Error 404" in response.content.decode())

    def test_user_can_access_to_data_if_owner(self):
        """test_user_can_access_a_data_if_owner

        Returns:

        """
        request = self.factory.get("core_main_app_xml_text_editor_view")
        request.GET = {"id": str(self.fixture.data_1.id)}
        request.user = self.user1
        response = DataContentEditor.as_view()(
            request,
        )
        self.assertEqual(response.status_code, 200)

    def test_user_can_format_xml_content(self):
        """test_user_can_format_xml_content

        Returns:

        """
        data = {
            "content": "<root><element>value2</element></root>",
            "action": "format",
            "id": str(self.fixture.data_1.id),
        }
        request = self.factory.post("core_main_app_xml_text_editor_view", data)

        request.user = self.user1
        response = DataContentEditor.as_view()(request)
        self.assertEqual(response.status_code, 200)

    def test_user_can_validate_xml_content(self):
        """test_user_can_validate_xml_content

        Returns:

        """
        data = {
            "content": "<root></root>",
            "action": "validate",
            "template_id": str(self.fixture.data_1.template.id),
            "id": str(self.fixture.data_1.id),
        }
        request = self.factory.post("core_main_app_xml_text_editor_view", data)

        request.user = self.user1
        response = DataContentEditor.as_view()(request)
        self.assertEqual(response.status_code, 200)

    def test_user_validate_bad_xml_content_returns_error(self):
        """test_user_validate_xml_content_returns_error

        Returns:

        """
        data = {
            "content": "<tag></tag>",
            "action": "validate",
            "template_id": str(self.fixture.data_1.template.id),
            "id": str(self.fixture.data_1.id),
        }
        request = self.factory.post("core_main_app_xml_text_editor_view", data)
        request.user = self.user1

        response = DataContentEditor.as_view()(request)
        self.assertEqual(response.status_code, 400)

    def test_user_validate_empty_content_returns_error(self):
        """test_user_validate_empty_content_returns_error

        Returns:

        """
        data = {
            "content": "",
            "action": "validate",
            "template_id": str(self.fixture.data_1.template.id),
            "id": str(self.fixture.data_1.id),
        }
        request = self.factory.post("core_main_app_xml_text_editor_view", data)

        request.user = self.user1
        response = DataContentEditor.as_view()(request)
        self.assertEqual(response.status_code, 400)

    def test_user_validate_xml_content_returns_error(self):
        """test_user_validate_xml_content_returns_error

        Returns:

        """
        data = {
            "content": "<root></root>",
            "action": "validate",
            "template_id": "",
            "id": str(self.fixture.data_1.id),
        }
        request = self.factory.post("core_main_app_xml_text_editor_view", data)
        request.user = self.user1
        response = DataContentEditor.as_view()(request)
        self.assertEqual(response.status_code, 400)

    def test_user_can_save_xml_content(self):
        """test_user_can_save_xml_content

        Returns:

        """
        data = {
            "content": "<root></root>",
            "action": "save",
            "document_id": str(self.fixture.data_1.id),
            "id": str(self.fixture.data_1.id),
        }
        request = self.factory.post("core_main_app_xml_text_editor_view", data)
        setattr(request, "session", "session")
        messages = FallbackStorage(request)
        setattr(request, "_messages", messages)
        request.user = self.user1
        response = DataContentEditor.as_view()(request)
        self.assertEqual(response.status_code, 200)

    def test_user_save_xml_content_returns_acl_error(self):
        """test_user_save_xml_content_returns_acl_error

        Returns:

        """
        data = {
            "content": "<root></root>",
            "action": "save",
            "document_id": str(self.fixture.data_1.id),
            "id": str(self.fixture.data_1.id),
        }
        request = self.factory.post("core_main_app_xml_text_editor_view", data)
        request.user = self.anonymous
        response = DataContentEditor.as_view()(request)
        self.assertEqual(response.status_code, 403)

    def test_user_save_xml_content_returns_dne_error(self):
        """test_user_save_xml_content_returns_dne_error

        Returns:

        """
        data = {
            "content": "<root></root>",
            "action": "save",
            "document_id": "-1",
            "id": "-1",
        }
        request = self.factory.post("core_main_app_xml_text_editor_view", data)
        request.user = self.user1
        response = DataContentEditor.as_view()(request)
        self.assertEqual(response.status_code, 400)

    def test_user_save_xml_content_returns_error(self):
        """test_user_save_xml_content_returns_error

        Returns:

        """
        data = {
            "action": "save",
            "document_id": "-1",
            "id": "-1",
        }
        request = self.factory.post("core_main_app_xml_text_editor_view", data)
        request.user = self.user1
        response = DataContentEditor.as_view()(request)
        self.assertEqual(response.status_code, 400)

    @patch("core_main_app.utils.file.get_byte_size_from_string")
    def test_xml_content_too_big_returns_error(self, mock_get_byte_size):
        """test_user_save_xml_content_returns_error

        Returns:

        """
        mock_get_byte_size.return_value = MAX_DOCUMENT_EDITING_SIZE + 1
        request = self.factory.get("core_main_app_xml_text_editor_view")
        request.GET = {"id": str(self.fixture.data_1.id)}
        request.user = self.user1
        response = DataContentEditor.as_view()(request)
        self.assertTrue(
            "MAX_DOCUMENT_EDITING_SIZE" in response.content.decode()
        )


class TestXSDTextEditorView(IntegrationBaseTestCase):
    """Test Post XSD Text Editor View"""

    def setUp(self):
        """setUp

        Returns:

        """
        self.factory = RequestFactory()
        self.user1 = create_mock_user(user_id="1")
        self.anonymous = AnonymousUser()
        self.fixture = AccessControlDataFixture()
        self.fixture.insert_data()

    def test_anonymous_user_can_not_access_template_content(self):
        """test_anonymous_user_can_not_access_template_content

        Returns:

        """
        request = self.factory.get("core_main_app_xsd_text_editor_view")
        request.user = self.anonymous
        request.GET = {"id": str(self.fixture.template.id)}
        response = XSDEditor.as_view()(request)

        self.assertTrue("Error 403" in response.content.decode())

    def test_user_can_not_access_template_if_not_found(self):
        """test_user_can_not_access_template_if_not_found

        Returns:

        """
        request = self.factory.get("core_main_app_xsd_text_editor_view")
        request.user = self.user1
        request.GET = {"id": "-1"}
        response = XSDEditor.as_view()(request)
        self.assertTrue("Error 404" in response.content.decode())

    def test_user_can_access_template(self):
        """test_user_can_access_template

        Returns:

        """
        request = self.factory.get("core_main_app_xsd_text_editor_view")
        request.user = self.user1
        request.GET = {"id": str(self.fixture.template.id)}
        response = XSDEditor.as_view()(request)
        self.assertEqual(response.status_code, 200)

    def test_user_can_format_xsd_content(self):
        """test_user_can_format_xsd_content

        Returns:

        """
        data = {
            "content": self.fixture.template.content,
            "action": "format",
            "id": str(self.fixture.template.id),
        }
        request = self.factory.post("core_main_app_xsd_text_editor_view", data)

        request.user = self.user1
        response = XSDEditor.as_view()(request)
        self.assertEqual(response.status_code, 200)

    def test_user_can_validate_xsd_content(self):
        """test_user_can_validate_xsd_content

        Returns:

        """
        data = {
            "content": self.fixture.template.content,
            "action": "validate",
            "template_id": str(self.fixture.template.id),
            "id": str(self.fixture.template.id),
        }
        request = self.factory.post("core_main_app_xsd_text_editor_view", data)
        request.user = self.user1
        response = XSDEditor.as_view()(request)
        self.assertEqual(response.status_code, 200)

    def test_user_validate_empty_xsd_content_returns_error(self):
        """test_user_validate_empty_xsd_content_returns_error

        Returns:

        """
        data = {
            "content": "",
            "action": "validate",
            "template_id": str(self.fixture.template.id),
            "id": str(self.fixture.template.id),
        }
        request = self.factory.post("core_main_app_xsd_text_editor_view", data)
        request.user = self.user1
        response = XSDEditor.as_view()(request)
        self.assertEqual(response.status_code, 400)

    def test_user_validate_bad_xsd_content_returns_error(self):
        """test_user_validate_bad_xsd_content_returns_error

        Returns:

        """
        xsd = (
            '<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">'
            '<xs:element nam="root"></xs:element></xs:schema>'
        )
        data = {
            "content": xsd,
            "action": "validate",
            "template_id": str(self.fixture.template.id),
            "id": str(self.fixture.template.id),
        }
        request = self.factory.post("core_main_app_xsd_text_editor_view", data)
        request.user = self.user1
        response = XSDEditor.as_view()(request)
        self.assertEqual(response.status_code, 400)

    def test_user_can_save_xsd_content(self):
        """test_user_can_save_xsd_content

        Returns:

        """
        data = {
            "content": self.fixture.template.content,
            "action": "save",
            "id": str(self.fixture.template.id),
            "document_id": str(self.fixture.template.id),
        }
        request = self.factory.post("core_main_app_xsd_text_editor_view", data)
        request.user = self.user1
        response = XSDEditor.as_view()(request)
        self.assertEqual(response.status_code, 200)

    def test_user_save_xsd_content_returns_acl_error(self):
        """test_user_save_xsd_content_returns_acl_error

        Returns:

        """
        data = {
            "content": self.fixture.template.content,
            "action": "save",
            "id": str(self.fixture.template.id),
            "document_id": str(self.fixture.template.id),
        }
        request = self.factory.post("core_main_app_xsd_text_editor_view", data)
        request.user = self.anonymous
        response = XSDEditor.as_view()(request)
        self.assertEqual(response.status_code, 403)

    def test_user_save_xml_content_returns_dne_error(self):
        """test_user_save_xsd_content_returns_dne_error

        Returns:

        """
        data = {
            "content": self.fixture.template.content,
            "action": "save",
            "id": "-1",
            "document_id": "-1",
        }
        request = self.factory.post("core_main_app_xsd_text_editor_view", data)
        request.user = self.user1
        response = XSDEditor.as_view()(request)
        self.assertEqual(response.status_code, 400)

    def test_user_save_xml_content_returns_error(self):
        """test_user_save_xml_content_returns_error

        Returns:

        """
        data = {
            "action": "save",
            "id": "-1",
            "document_id": "-1",
        }
        request = self.factory.post("core_main_app_xsd_text_editor_view", data)
        request.user = self.user1
        response = XSDEditor.as_view()(request)
        self.assertEqual(response.status_code, 400)


class TestViewBlob(IntegrationBaseTestCase):
    """TestViewBlob"""

    def setUp(self):
        """setUp

        Returns:

        """
        self.factory = RequestFactory()
        self.user1 = create_mock_user(user_id="1")
        self.user2 = create_mock_user(user_id="2")
        self.anonymous = AnonymousUser()
        self.fixture = AccessControlBlobWithMetadataFixture()
        self.fixture.insert_data()

    def test_a_user_can_access_a_blob_if_owner(self):
        """test_a_user_can_access_a_blob_if_owner

        Returns:

        """
        request = self.factory.get("core_main_app_blob_detail")
        request.user = self.user1
        request.GET = {"id": str(self.fixture.blob_1.id)}
        response = ViewBlob.as_view()(request)
        self.assertTrue(
            self.fixture.blob_1.filename in response.content.decode()
        )
        self.assertTrue("Manage Metadata" in response.content.decode())

    def test_a_user_access_a_blob_with_wrong_id_returns_error(self):
        """test_a_user_access_a_blob_with_wrong_id_returns_error

        Returns:

        """
        request = self.factory.get("core_main_app_blob_detail")
        request.user = self.user1
        request.GET = {"id": "1234"}
        response = ViewBlob.as_view()(request)
        self.assertTrue("Error 404" in response.content.decode())

    @patch("core_main_app.components.blob.api.get_by_id")
    def test_a_user_access_a_blob_with_exception_returns_error(
        self, mock_get_by_id
    ):
        """test_a_user_access_a_blob_with_exception_returns_error

        Returns:

        """
        request = self.factory.get("core_main_app_blob_detail")
        request.user = self.user1
        request.GET = {"id": str(self.fixture.blob_1.id)}
        mock_get_by_id.side_effect = Exception()
        response = ViewBlob.as_view()(request)
        self.assertTrue("Error 400" in response.content.decode())

    @patch("core_main_app.access_control.api.check_can_write")
    def test_a_user_can_not_manage_metadata_if_no_write_perm(
        self, mock_check_can_write
    ):
        """test_a_user_can_not_manage_metadata_if_no_write_perm

        Returns:

        """
        request = self.factory.get("core_main_app_blob_detail")
        request.user = self.user2
        mock_check_can_write.side_effect = AccessControlError("Forbidden")
        request.GET = {"id": str(self.fixture.blob_2.id)}
        response = ViewBlob.as_view()(request)
        self.assertTrue("Error" not in response.content.decode())
        self.assertTrue("Manage Metadata" not in response.content.decode())

    def test_an_anonymous_user_can_not_access_a_blob_that_is_not_in_a_workspace(
        self,
    ):
        """test_an_anonymous_user_can_not_access_a_blob_that_is_not_in_a_workspace

        Returns:

        """
        request = self.factory.get("core_main_app_blob_detail")
        request.user = self.anonymous
        request.GET = {"id": str(self.fixture.blob_1.id)}
        response = ViewBlob.as_view()(request)
        self.assertTrue(
            self.fixture.blob_1.filename not in response.content.decode()
        )
        self.assertTrue("Error 403" in response.content.decode())

    def test_an_anonymous_user_can_not_access_a_blob_that_is_in_a_private_workspace(
        self,
    ):
        """test_an_anonymous_user_can_not_access_a_blob_that_is_in_a_private_workspace

        Returns:

        """
        request = self.factory.get("core_main_app_blob_detail")
        request.user = self.anonymous
        request.GET = {"id": str(self.fixture.blob_2.id)}
        response = ViewBlob.as_view()(request)
        self.assertTrue(
            self.fixture.blob_1.filename not in response.content.decode()
        )
        self.assertTrue("Error 403" in response.content.decode())


class TestManageBlobMetadata(IntegrationBaseTestCase):
    """TestManageBlobMetadata"""

    def setUp(self):
        """setUp

        Returns:

        """
        self.factory = RequestFactory()
        self.user1 = create_mock_user(user_id="1")
        self.user2 = create_mock_user(user_id="2")
        self.anonymous = AnonymousUser()
        self.fixture = AccessControlBlobWithMetadataFixture()
        self.fixture.insert_data()

    def test_a_user_can_access_blob_metadata_if_owner(self):
        """test_a_user_can_access_blob_metadata_if_owner

        Returns:

        """
        request = self.factory.get("core_main_app_blob_metadata")
        request.user = self.user1
        response = ManageBlobMetadata.as_view()(
            request, str(self.fixture.blob_1.id)
        )
        self.assertTrue(
            self.fixture.blob_1.filename in response.content.decode()
        )

    def test_a_user_manage_metadata_with_wrong_id_returns_error(self):
        """test_a_user_manage_metadata_with_wrong_id_returns_error

        Returns:

        """
        request = self.factory.get("core_main_app_blob_metadata")
        request.user = self.user1
        response = ManageBlobMetadata.as_view()(request, "1234")
        self.assertTrue("Error 404" in response.content.decode())

    @patch("core_main_app.components.blob.api.get_by_id")
    def test_a_user_manage_metadata_with_exception_returns_error(
        self, mock_get_by_id
    ):
        """test_a_user_manage_metadata_with_exception_returns_error

        Returns:

        """
        request = self.factory.get("core_main_app_blob_metadata")
        request.user = self.user1
        mock_get_by_id.side_effect = Exception()
        response = ManageBlobMetadata.as_view()(
            request, str(self.fixture.blob_1.id)
        )
        self.assertTrue("Error 400" in response.content.decode())

    @patch("core_main_app.access_control.api.check_can_write")
    def test_a_user_can_not_manage_metadata_if_no_write_perm(
        self, mock_check_can_write
    ):
        """test_a_user_can_not_manage_metadata_if_no_write_perm

        Returns:

        """
        request = self.factory.get("core_main_app_blob_metadata")
        request.user = self.user2
        mock_check_can_write.side_effect = AccessControlError("Forbidden")
        response = ManageBlobMetadata.as_view()(
            request, str(self.fixture.blob_2.id)
        )
        self.assertTrue("Error" not in response.content.decode())

    def test_an_anonymous_user_can_not_access_blob_metadata_if_blob_not_in_a_workspace(
        self,
    ):
        """test_an_anonymous_user_can_not_access_blob_metadata_if_blob_not_in_a_workspace

        Returns:

        """
        request = self.factory.get("core_main_app_blob_metadata")
        request.user = self.anonymous
        response = ManageBlobMetadata.as_view()(
            request, str(self.fixture.blob_1.id)
        )
        self.assertTrue(
            self.fixture.blob_1.filename not in response.content.decode()
        )
        self.assertTrue("Error 403" in response.content.decode())

    def test_an_anonymous_user_can_not_access_blob_that_is_in_a_private_workspace(
        self,
    ):
        """test_an_anonymous_user_can_not_access_blob_that_is_in_a_private_workspace

        Returns:

        """
        request = self.factory.get("core_main_app_blob_metadata")
        request.user = self.anonymous
        response = ManageBlobMetadata.as_view()(
            request, str(self.fixture.blob_2.id)
        )
        self.assertTrue(
            self.fixture.blob_1.filename not in response.content.decode()
        )
        self.assertTrue("Error 403" in response.content.decode())


class TestLoadBlobMetadataForm(IntegrationBaseTestCase):
    """TestLoadBlobMetadataForm"""

    def setUp(self):
        """setUp

        Returns:

        """
        self.factory = RequestFactory()
        self.user1 = create_mock_user(user_id="1")
        self.anonymous = AnonymousUser()
        self.fixture = AccessControlBlobWithMetadataFixture()
        self.fixture.insert_data()

    def test_an_anonymous_user_can_not_load_blob_metadata_form(self):
        """test_an_anonymous_user_can_not_load_blob_metadata_form

        Returns:

        """
        request = self.factory.get("core_main_blob_metadata_form")
        request.user = self.anonymous
        request.GET = {"id": str(self.fixture.blob_1)}
        response = LoadBlobMetadataForm.as_view()(request)
        self.assertEqual(response.status_code, 302)

    @patch("core_main_app.components.data.api.get_all_by_user")
    def test_user_can_load_blob_metadata_form(self, mock_get_all_by_user):
        """test_user_can_load_blob_metadata_form

        Returns:

        """
        request = self.factory.get("core_main_blob_metadata_form")
        request.user = self.user1
        request.GET = {"blob_id": str(self.fixture.blob_1.id)}
        mock_get_all_by_user.return_value = []
        response = LoadBlobMetadataForm.as_view()(request)
        self.assertEqual(response.status_code, 200)
        self.assertTrue("form" in response.content.decode())

    @patch(
        "core_main_app.components.workspace.api.get_all_workspaces_with_read_access_by_user"
    )
    @patch("core_main_app.components.data.api.get_all_by_user")
    def test_user_can_load_blob_metadata_form_with_list_of_data(
        self, mock_get_all_by_user, get_all_workspaces_with_read_access_by_user
    ):
        """test_user_can_load_blob_metadata_form_with_list_of_data

        Returns:

        """
        request = self.factory.get("core_main_blob_metadata_form")
        request.user = self.user1
        request.GET = {"blob_id": str(self.fixture.blob_1.id)}
        mock_get_all_by_user.return_value = [self.fixture.data_4]
        get_all_workspaces_with_read_access_by_user.return_value = [
            self.fixture.workspace_1
        ]
        response = LoadBlobMetadataForm.as_view()(request)
        self.assertEqual(response.status_code, 200)
        self.assertTrue("form" in response.content.decode())
        self.assertTrue(self.fixture.data_4.title in response.content.decode())

    @patch("core_main_app.components.blob.api.get_by_id")
    def test_user_can_load_blob_metadata_form_with_model_error(
        self, mock_get_by_id
    ):
        """test_user_can_load_blob_metadata_form_with_model_error

        Returns:

        """
        request = self.factory.get("core_main_blob_metadata_form")
        request.user = self.user1
        request.GET = {"blob_id": str(self.fixture.blob_1.id)}
        mock_get_by_id.side_effect = ModelError("Error")
        response = LoadBlobMetadataForm.as_view()(request)
        self.assertEqual(response.status_code, 400)

    @patch("core_main_app.components.blob.api.get_by_id")
    def test_user_can_load_blob_metadata_form_with_exception(
        self, mock_get_by_id
    ):
        """test_user_can_load_blob_metadata_form_with_exception

        Returns:

        """
        request = self.factory.get("core_main_blob_metadata_form")
        request.user = self.user1
        request.GET = {"blob_id": str(self.fixture.blob_1.id)}
        mock_get_by_id.side_effect = Exception("Error")
        response = LoadBlobMetadataForm.as_view()(request)
        self.assertEqual(response.status_code, 400)


class TestAddMetadataToBlob(IntegrationBaseTestCase):
    """TestAddMetadataToBlob"""

    def setUp(self):
        """setUp

        Returns:

        """
        self.factory = RequestFactory()
        self.user1 = create_mock_user(user_id="1")
        self.anonymous = AnonymousUser()
        self.fixture = AccessControlBlobWithMetadataFixture()
        self.fixture.insert_data()

    def test_an_anonymous_user_can_not_add_metadata_to_blob(self):
        """test_an_anonymous_user_can_not_add_metadata_to_blob

        Returns:

        """
        request = self.factory.post("core_main_blob_add_metadata")
        request.user = self.anonymous
        request.POST = {"id": str(self.fixture.blob_1)}
        response = AddMetadataToBlob.as_view()(request)
        self.assertEqual(response.status_code, 302)

    def test_user_add_metadata_to_blob(self):
        """test_user_add_metadata_to_blob

        Returns:

        """
        request = self.factory.post("core_main_blob_metadata_form")
        request.user = self.user1
        request.POST = QueryDict("").copy()
        request.POST.update({"blob_id": str(self.fixture.blob_1.id)})
        # Add middlewares
        mock_get_response = MagicMock()
        middleware = SessionMiddleware(mock_get_response)
        middleware.process_request(request)
        middleware = MessageMiddleware(mock_get_response)
        middleware.process_request(request)
        request.session.save()
        response = AddMetadataToBlob.as_view()(request)
        self.assertEqual(response.status_code, 200)

    def test_user_add_metadata_to_blob_with_list_of_ids(self):
        """test_user_add_metadata_to_blob

        Returns:

        """
        request = self.factory.post("core_main_blob_metadata_form")
        request.user = self.user1
        request.POST = QueryDict("").copy()
        request.POST.update({"blob_id": str(self.fixture.blob_1.id)})
        request.POST.update({"metadata_id[]": str(self.fixture.data_4.id)})
        # Add middlewares
        mock_get_response = MagicMock()
        middleware = SessionMiddleware(mock_get_response)
        middleware.process_request(request)
        middleware = MessageMiddleware(mock_get_response)
        middleware.process_request(request)
        request.session.save()
        response = AddMetadataToBlob.as_view()(request)
        self.assertEqual(response.status_code, 200)

    @patch("core_main_app.components.blob.api.get_by_id")
    def test_user_can_not_add_metadata_to_blob_with_model_error(
        self, mock_get_by_id
    ):
        """test_user_can_not_add_metadata_to_blob_with_model_error

        Returns:

        """
        request = self.factory.post("core_main_blob_add_metadata")
        request.user = self.user1
        request.POST = QueryDict("").copy()
        request.POST.update({"blob_id": str(self.fixture.blob_1.id)})
        mock_get_by_id.side_effect = ModelError("Error")
        response = AddMetadataToBlob.as_view()(request)
        self.assertEqual(response.status_code, 400)
        self.assertEqual(response.content, b"Blob not found.")

    @patch("core_main_app.components.blob.api.get_by_id")
    def test_user_can_not_add_metadata_to_blob_with_acl_error(
        self, mock_get_by_id
    ):
        """test_user_can_not_add_metadata_to_blob_with_model_error

        Returns:

        """
        request = self.factory.post("core_main_blob_add_metadata")
        request.user = self.user1
        request.POST = QueryDict("").copy()
        request.POST.update({"blob_id": str(self.fixture.blob_1.id)})
        mock_get_by_id.side_effect = AccessControlError("Error")
        response = AddMetadataToBlob.as_view()(request)
        self.assertEqual(response.status_code, 400)
        self.assertEqual(response.content, b"Permission denied.")

    @patch("core_main_app.components.blob.api.get_by_id")
    def test_user_can_not_add_metadata_to_blob_with_model_exception(
        self, mock_get_by_id
    ):
        """test_user_can_not_add_metadata_to_blob_with_model_exception

        Returns:

        """
        request = self.factory.post("core_main_blob_add_metadata")
        request.user = self.user1
        request.POST = QueryDict("").copy()
        request.POST.update({"blob_id": str(self.fixture.blob_1.id)})
        mock_get_by_id.side_effect = Exception("Error")
        response = AddMetadataToBlob.as_view()(request)
        self.assertEqual(response.status_code, 400)
        self.assertEqual(response.content, b"An unexpected error occurred.")


class TestRemoveMetadataFromBlob(IntegrationBaseTestCase):
    """TestRemoveMetadataFromBlob"""

    def setUp(self):
        """setUp

        Returns:

        """
        self.factory = RequestFactory()
        self.user1 = create_mock_user(user_id="1")
        self.anonymous = AnonymousUser()
        self.fixture = AccessControlBlobWithMetadataFixture()
        self.fixture.insert_data()

    def test_an_anonymous_user_can_not_remove_metadata_from_blob(self):
        """test_an_anonymous_user_can_not_remove_metadata_from_blob

        Returns:

        """
        request = self.factory.post("core_main_blob_remove_metadata")
        request.user = self.anonymous
        request.POST = {"id": str(self.fixture.blob_1)}
        response = RemoveMetadataFromBlob.as_view()(request)
        self.assertEqual(response.status_code, 302)

    def test_user_remove_metadata_from_blob(self):
        """test_user_remove_metadata_from_blob

        Returns:

        """
        request = self.factory.post("core_main_blob_remove_metadata")
        request.user = self.user1
        request.POST = QueryDict("").copy()
        request.POST.update({"blob_id": str(self.fixture.blob_1.id)})
        request.POST.update({"metadata_id": str(self.fixture.data_1.id)})
        # Add middlewares
        mock_get_response = MagicMock()
        middleware = SessionMiddleware(mock_get_response)
        middleware.process_request(request)
        middleware = MessageMiddleware(mock_get_response)
        middleware.process_request(request)
        request.session.save()
        response = RemoveMetadataFromBlob.as_view()(request)
        self.assertEqual(response.status_code, 200)

    @patch("core_main_app.components.blob.api.get_by_id")
    def test_user_can_not_remove_metadata_to_blob_with_model_error(
        self, mock_get_by_id
    ):
        """test_user_can_not_remove_metadata_to_blob_with_model_error

        Returns:

        """
        request = self.factory.post("core_main_blob_remove_metadata")
        request.user = self.user1
        request.POST = QueryDict("").copy()
        request.POST.update({"blob_id": str(self.fixture.blob_1.id)})
        mock_get_by_id.side_effect = ModelError("Error")
        response = RemoveMetadataFromBlob.as_view()(request)
        self.assertEqual(response.status_code, 400)
        self.assertEqual(response.content, b"Blob not found.")

    @patch("core_main_app.components.blob.api.get_by_id")
    def test_user_can_not_remove_metadata_to_blob_with_acl_error(
        self, mock_get_by_id
    ):
        """test_user_can_not_remove_metadata_to_blob_with_acl_error

        Returns:

        """
        request = self.factory.post("core_main_blob_remove_metadata")
        request.user = self.user1
        request.POST = QueryDict("").copy()
        request.POST.update({"blob_id": str(self.fixture.blob_1.id)})
        mock_get_by_id.side_effect = AccessControlError("Error")
        response = RemoveMetadataFromBlob.as_view()(request)
        self.assertEqual(response.status_code, 400)
        self.assertEqual(response.content, b"Permission denied.")

    @patch("core_main_app.components.blob.api.get_by_id")
    def test_user_can_not_remove_metadata_to_blob_with_model_exception(
        self, mock_get_by_id
    ):
        """test_user_can_not_remove_metadata_to_blob_with_model_exception

        Returns:

        """
        request = self.factory.post("core_main_blob_remove_metadata")
        request.user = self.user1
        request.POST = QueryDict("").copy()
        request.POST.update({"blob_id": str(self.fixture.blob_1.id)})
        mock_get_by_id.side_effect = Exception("Error")
        response = RemoveMetadataFromBlob.as_view()(request)
        self.assertEqual(response.status_code, 400)
        self.assertEqual(response.content, b"An unexpected error occurred.")


class TestUploadFile(IntegrationBaseTestCase):
    """TestUploadFile"""

    def setUp(self):
        """setUp

        Returns:

        """
        self.factory = RequestFactory()
        self.user1 = create_mock_user(user_id="1")
        self.anonymous = AnonymousUser()
        self.fixture = AccessControlBlobWithMetadataFixture()
        self.fixture.insert_data()
        self.file = SimpleUploadedFile(name="test.txt", content=b"test")

    def test_an_anonymous_user_can_not_upload_file(self):
        """test_an_anonymous_user_can_not_upload_file

        Returns:

        """
        request = self.factory.post("core_main_upload_file")
        request.user = self.anonymous
        request.FILES["file"] = self.file
        response = UploadFile.as_view()(request)
        self.assertEqual(response.status_code, 302)

    def test_user_can_upload_file(self):
        """test_user_can_upload_file

        Returns:

        """
        request = self.factory.post("core_main_upload_file")
        request.user = self.user1
        request.FILES["file"] = self.file
        mock_get_response = MagicMock()
        # Add middlewares
        middleware = SessionMiddleware(mock_get_response)
        middleware.process_request(request)
        middleware = MessageMiddleware(mock_get_response)
        middleware.process_request(request)
        response = UploadFile.as_view()(request)
        self.assertEqual(response.status_code, 200)

    @patch("core_main_app.components.blob.api.insert")
    def test_user_can_not_upload_file_with_error(self, mock_insert):
        """test_user_can_not_upload_file_with_error

        Returns:

        """
        request = self.factory.post("core_main_upload_file")
        request.user = self.user1
        request.FILES["file"] = self.file
        mock_insert.side_effect = Exception()
        # Add middlewares
        mock_get_response = MagicMock()
        middleware = SessionMiddleware(mock_get_response)
        middleware.process_request(request)
        middleware = MessageMiddleware(mock_get_response)
        middleware.process_request(request)
        response = UploadFile.as_view()(request)
        self.assertEqual(response.status_code, 400)

    def test_user_can_not_upload_file_with_invalid_form(self):
        """test_user_can_not_upload_file_with_invalid_form

        Returns:

        """
        request = self.factory.post("core_main_upload_file")
        request.user = self.user1
        request.FILES["file"] = 123
        # Add middlewares
        mock_get_response = MagicMock()
        middleware = SessionMiddleware(mock_get_response)
        middleware.process_request(request)
        middleware = MessageMiddleware(mock_get_response)
        middleware.process_request(request)
        response = UploadFile.as_view()(request)
        self.assertEqual(response.status_code, 400)



""" Integration Test for Workspace API
"""
from tests.components.user.fixtures.fixtures import UserFixtures
from tests.components.workspace.fixtures.fixtures import WorkspaceFixtures

from core_main_app.commons import exceptions
from core_main_app.components.data.models import Data
from core_main_app.components.template.models import Template
from core_main_app.components.workspace import api as workspace_api
from core_main_app.utils.integration_tests.integration_base_transaction_test_case import (
    IntegrationTransactionTestCase,
)

TITLE_1 = "title 1"


class TestCreateWorkspace(IntegrationTransactionTestCase):
    """TestCreateWorkspace"""

    def test_create_one_workspace(self):
        """test create one workspace

        Returns:

        """
        # Context
        user1 = UserFixtures().create_user()
        # Act
        workspace = workspace_api.create_and_save(TITLE_1, user1.id)
        # Assert
        self.assertEqual(workspace.title, TITLE_1)
        self.assertEqual(workspace.owner, str(user1.id))

    def test_create_two_workspaces_same_name_same_user(self):
        """test create two workspaces same name same user

        Returns:

        """
        # Context
        user1 = UserFixtures().create_user()
        # Act
        workspace_api.create_and_save(TITLE_1, user1.id)
        with self.assertRaises(exceptions.NotUniqueError):
            workspace_api.create_and_save(TITLE_1, user1.id)

    def test_create_two_workspaces_same_name_different_case_same_user(self):
        """test create two workspaces same name different case same user

        Returns:

        """
        # Context
        user1 = UserFixtures().create_user()
        # Act
        workspace_api.create_and_save(TITLE_1, user1.id)
        with self.assertRaises(exceptions.NotUniqueError):
            workspace_api.create_and_save(TITLE_1.upper(), user1.id)

    def test_create_two_workspaces_same_name_different_user(self):
        """test create two workspaces same name different user

        Returns:

        """
        # Context
        number_workspace = len(workspace_api.get_all())
        user1 = UserFixtures().create_user(username="user1")
        user2 = UserFixtures().create_user(username="user2")
        # Act
        workspace_api.create_and_save(TITLE_1, user1.id)
        workspace_api.create_and_save(TITLE_1, user2.id)
        # Assert
        self.assertEqual(number_workspace + 2, len(workspace_api.get_all()))

    def test_create_workspace_without_owner(self):
        """test create workspace without owner

        Returns:

        """
        # Act
        workspace = workspace_api.create_and_save(TITLE_1)
        # Assert
        self.assertEqual(workspace.title, TITLE_1)
        self.assertIsNone(workspace.owner)


class TestIsWorkspaceGlobal(IntegrationTransactionTestCase):
    """Test Is Workspace Global"""

    def test_is_workspace_global_true(self):
        """test is workspace global true

        Returns:

        """
        # Context
        workspace = WorkspaceFixtures.create_global_workspace(TITLE_1)
        # Act
        result = workspace_api.is_workspace_global(workspace)
        # Assert
        self.assertTrue(result)

    def test_is_workspace_global_false(self):
        """test is workspace global false

        Returns:

        """
        # Context
        workspace = WorkspaceFixtures.create_workspace("1", TITLE_1)
        # Act
        result = workspace_api.is_workspace_global(workspace)
        # Assert
        self.assertFalse(result)


class TestGetGlobalWorkspace(IntegrationTransactionTestCase):
    """Test Get Global Workspace"""

    def test_get_global_workspace(self):
        """test get global workspace

        Returns:

        """
        # Context
        workspace = WorkspaceFixtures.create_global_workspace(TITLE_1)
        # Act
        result = workspace_api.get_global_workspace()
        # Assert
        self.assertEqual(workspace, result)

    def test_get_global_workspace_multiple_workspace(self):
        """test get global workspace multiple workspace

        Returns:

        """
        # Context
        global_workspace = WorkspaceFixtures.create_global_workspace(TITLE_1)
        WorkspaceFixtures.create_workspace("1", TITLE_1)
        # Act
        result = workspace_api.get_global_workspace()
        # Assert
        self.assertEqual(global_workspace, result)

    def test_get_global_workspace_not_global(self):
        """test get global workspace not global

        Returns:

        """
        # Context
        WorkspaceFixtures.create_workspace("1", TITLE_1)
        # Act
        with self.assertRaises(exceptions.DoesNotExist):
            workspace_api.get_global_workspace()


class TestCheckIfWorkspaceCanBeChanged(IntegrationTransactionTestCase):
    """Test Check If Workspace Can Be Changed"""

    def test_workspace_can_be_changed_return_true_if_workspace_is_none_and_allow_public_is_false(
        self,
    ):
        """test workspace can be changed return true if workspace is none and allow public is false

        Returns:

        """
        # A global workspace is public
        data = Data(
            template=Template(),
            user_id="1",
            dict_content=None,
            title="title",
            workspace=None,
        )

        # Act
        result = workspace_api.check_if_workspace_can_be_changed(data, False)
        # Assert
        self.assertEqual(result, True)

    def test_workspace_can_be_changed_return_true_if_workspace_is_none_and_allow_public_is_true(
        self,
    ):
        """test workspace can be changed return true if workspace is none and allow public is false

        Returns:

        """
        # A global workspace is public
        data = Data(
            template=Template(),
            user_id="1",
            dict_content=None,
            title="title",
            workspace=None,
        )

        # Act
        result = workspace_api.check_if_workspace_can_be_changed(data, True)
        # Assert
        self.assertEqual(result, True)

    def test_workspace_can_be_changed_return_false_if_workspace_is_public_and_allow_public_is_false(
        self,
    ):
        """test workspace can be changed return false if workspace is public
        and allow public is false

        Returns:

        """
        # A global workspace is public
        workspace = WorkspaceFixtures.create_global_workspace(TITLE_1)
        data = Data(
            template=Template(),
            user_id="1",
            dict_content=None,
            title="title",
            workspace=workspace,
        )

        # Act
        result = workspace_api.check_if_workspace_can_be_changed(data, False)
        # Assert
        self.assertEqual(result, False)

    def test_workspace_can_be_changed_return_true_if_workspace_is_public_and_allow_public_is_true(
        self,
    ):
        """test workspace can be changed return true if workspace is public and allow public is true

        Returns:

        """
        # A global workspace is public
        workspace = WorkspaceFixtures.create_global_workspace(TITLE_1)
        data = Data(
            template=Template(),
            user_id="1",
            dict_content=None,
            title="title",
            workspace=workspace,
        )
        # Act
        result = workspace_api.check_if_workspace_can_be_changed(data, True)
        # Assert
        self.assertEqual(result, True)



""" Blob model
"""
from django.contrib.auth.models import User
from django.core.exceptions import ObjectDoesNotExist
from django.core.validators import RegexValidator
from django.db import models

from core_main_app.commons import exceptions
from core_main_app.commons.regex import NOT_EMPTY_OR_WHITESPACES
from core_main_app.components.blob.access_control import (
    filter_accessible_metadata,
)
from core_main_app.components.workspace.models import Workspace
from core_main_app.settings import CHECKSUM_ALGORITHM
from core_main_app.utils.checksum import compute_checksum
from core_main_app.utils.storage.storage import (
    user_directory_path,
    core_file_storage,
)


class Blob(models.Model):
    """Blob object"""

    filename = models.CharField(
        blank=False,
        validators=[
            RegexValidator(
                regex=NOT_EMPTY_OR_WHITESPACES,
                message="Filename must not be empty or only whitespaces",
                code="invalid_title",
            ),
        ],
        max_length=200,
    )
    user_id = models.CharField(blank=False, max_length=200)
    workspace = models.ForeignKey(
        Workspace, on_delete=models.SET_NULL, blank=True, null=True
    )
    blob = models.FileField(
        null=True,
        max_length=250,
        upload_to=user_directory_path,
        storage=core_file_storage(model="blob"),
    )
    checksum = models.CharField(
        max_length=512, blank=True, default=None, null=True
    )

    creation_date = models.DateTimeField(auto_now_add=True)

    class Meta:
        ordering = ["-creation_date"]

    @property
    def owner_name(self):
        """Get owner name

        Returns:

        """
        return User.objects.get(pk=self.user_id).username

    def metadata(self, user):
        """Get blob metadata

        Args:
            user:

        Returns:

        """
        # Access _metadata with ACL check
        return filter_accessible_metadata(self._metadata.all(), user)

    @staticmethod
    def get_by_id(blob_id):
        """Return the object with the given id.

        Args:
            blob_id:

        Returns:
            Blob

        """
        try:
            return Blob.objects.get(pk=blob_id)
        except ObjectDoesNotExist as e:
            raise exceptions.DoesNotExist(str(e))
        except Exception as ex:
            raise exceptions.ModelError(str(ex))

    @staticmethod
    def get_all():
        """Return all blobs.

        Args:

        Returns:
            List of Blob instances.

        """
        return Blob.objects.all()

    @staticmethod
    def get_all_by_user_id(user_id):
        """Return all blobs by user.

        Args:
            user_id: User id.

        Returns:
            List of Blob instances for the given user id.

        """
        return Blob.objects.filter(user_id=str(user_id)).all()

    @staticmethod
    def get_all_by_workspace(workspace):
        """Get all blobs that belong to the workspace.

        Args:
            workspace:

        Returns:

        """
        return Blob.objects.filter(workspace=workspace).all()

    @staticmethod
    def get_all_by_list_workspace(list_workspace):
        """Get all blobs that belong to the list of workspace.

        Args:
            list_workspace:

        Returns:

        """
        return Blob.objects.filter(workspace__in=list_workspace).all()

    @staticmethod
    def get_none():
        """Return None object, used by blobs.

        Returns:

        """
        return Blob.objects.none()

    def __str__(self):
        """Blob object as string

        Returns:

        """
        return self.filename

    def save_object(self):
        """Custom save.

        Returns:
            Saved Instance.

        """
        try:
            if self.blob.file and CHECKSUM_ALGORITHM:
                self.checksum = compute_checksum(
                    self.blob.file, CHECKSUM_ALGORITHM
                )
            return self.save()
        except Exception as ex:
            raise exceptions.ModelError(str(ex))



""" Int Test Lock
"""

from core_main_app.commons.exceptions import LockError
from core_main_app.utils.tests_tools.MockUser import create_mock_user

from tests.components.data.fixtures.fixtures import DataFixtures
from core_main_app.utils.integration_tests.integration_base_test_case import (
    IntegrationBaseTestCase,
)
from core_main_app.components.lock import api as lock_api


fixture_data = DataFixtures()


class TestLockIsObjectLocked(IntegrationBaseTestCase):
    """Test Lock Is Object Locked"""

    fixture = fixture_data
    user1 = create_mock_user("1")
    user2 = create_mock_user("2")

    def test_is_object_locked_returns_true_when_locked(self):
        """test is object locked returns true when locked

        Returns:

        """
        # Arrange
        lock_api.set_lock_object(self.fixture.data_1, self.user1)

        # Act
        result = lock_api.is_object_locked(self.fixture.data_1, self.user2)

        # Assert
        self.assertEqual(result, True)

    def test_is_object_locked_returns_false_when_unlocked(self):
        """test is object locked returns false when unlocked

        Returns:

        """

        # Act
        result = lock_api.is_object_locked(self.fixture.data_1, self.user1)

        # Assert
        self.assertEqual(result, False)


class TestLockSetLock(IntegrationBaseTestCase):
    """Test Set Lock"""

    fixture = fixture_data
    user1 = create_mock_user("1")
    user2 = create_mock_user("2")

    def test_set_lock_locks_object_when_object_is_unlocked(self):
        """test set lock locks object when object is unlocked

        Returns:

        """

        # Act
        lock_api.set_lock_object(self.fixture.data_1, self.user1)

        # Assert
        self.assertEqual(
            lock_api.is_object_locked(self.fixture.data_1, self.user2), True
        )

    def test_set_lock_keeps_object_locked(self):
        """test set lock keeps object locked

        Returns:

        """
        # Arrange
        lock_api.set_lock_object(self.fixture.data_1, self.user1)

        # Act
        lock_api.set_lock_object(self.fixture.data_1, self.user1)

        # Assert
        self.assertEqual(
            lock_api.is_object_locked(self.fixture.data_1, self.user2), True
        )

    def test_set_lock_on_object_already_locked_by_another_user_raises_lock_error(
        self,
    ):
        """test set lock on object already locked by another user raises lock error

        Returns:

        """
        # Arrange
        lock_api.set_lock_object(self.fixture.data_1, self.user1)

        # Act  # Assert
        with self.assertRaises(LockError):
            lock_api.set_lock_object(self.fixture.data_1, self.user2)


class TestLockRemoveLockOnObject(IntegrationBaseTestCase):
    """Test Remove Lock On Object"""

    fixture = fixture_data
    user1 = create_mock_user("1")
    user2 = create_mock_user("2")

    def test_remove_lock_unlocks_object_when_locked(self):
        """test remove lock unlocks object when locked

        Returns:

        """
        # Arrange
        lock_api.set_lock_object(self.fixture.data_1, self.user1)

        # Act
        lock_api.remove_lock_on_object(self.fixture.data_1, self.user1)

        # Assert
        self.assertEqual(
            lock_api.is_object_locked(self.fixture.data_1, self.user2), False
        )

    def test_remove_lock_keeps_object_unlocked(self):
        """test set lock keeps object locked

        Returns:

        """
        # Act
        lock_api.remove_lock_on_object(self.fixture.data_1, self.user1)

        # Assert
        self.assertEqual(
            lock_api.is_object_locked(self.fixture.data_1, self.user2), False
        )

    def test_remove_lock_different_user_keeps_object_locked(self):
        """test set lock different user keeps object locked

        Returns:

        """
        # Arrange
        lock_api.set_lock_object(self.fixture.data_1, self.user1)

        # Act
        lock_api.remove_lock_on_object(self.fixture.data_1, self.user2)

        # Assert
        self.assertEqual(
            lock_api.is_object_locked(self.fixture.data_1, self.user2), True
        )



""" GridFSStorage class (previously blob_utils)
"""

from django.core.files.storage import Storage
from django.utils.deconstruct import deconstructible

from core_main_app.settings import (
    MONGO_DB,
    GRIDFS_STORAGE,
)

if GRIDFS_STORAGE:
    from gridfs import NoFile, GridFS

    @deconstructible
    class GridFSStorage(Storage):
        """GridFS Storage.

        Previously implemented using https://github.com/usnistgov/blob_utils/blob/master/blob_utils/gridfs_blob_host.py
        Now based on https://github.com/django-nonrel/mongodb-engine/blob/master/django_mongodb_engine/storage.py
        """

        def __init__(self, collection="fs"):
            """Initialize GridFSStorage

            Args:
                collection:
            """
            self.collection = collection

        def _open(self, name, mode="rb"):
            """Open file in read mode if exists, create a new one otherwise.

            Args:
                name:
                mode:

            Returns:

            """
            grid_fs = self._get_gridfs()
            try:
                return grid_fs.get_last_version(name)
            except NoFile:
                if "w" in mode:
                    return grid_fs.new_file(filename=name)
                else:
                    raise

        def _save(self, name, content):
            """Save the file

            Args:
                name:
                content:

            Returns:

            """
            grid_fs = self._get_gridfs()
            grid_fs.put(content, filename=name)
            return name

        def delete(self, name):
            """Delete the file

            Args:
                name:

            Returns:

            """
            grid_fs = self._get_gridfs()
            try:
                grid_fs.delete(grid_fs.get_last_version(filename=name)._id)
            except NoFile:
                pass

        def exists(self, name):
            """Check if file exists

            Args:
                name:

            Returns:

            """
            grid_fs = self._get_gridfs()
            return grid_fs.exists(filename=name)

        def size(self, name):
            """Return size of the file

            Args:
                name:

            Returns:

            """
            grid_fs = self._get_gridfs()
            return grid_fs.get_last_version(filename=name).length

        def url(self, name):
            """Url to access file (return filename to make django dashboard work).

            Args:
                name:

            Returns:
                filename to be displayed in Django dashboard

            """
            return name

        def get_created_time(self, name):
            """Get creation time

            Args:
                name:

            Returns:

            """
            grid_fs = self._get_gridfs()
            return grid_fs.get_last_version(filename=name).upload_date

        def _get_gridfs(self):
            """Get gridFS connection

            Returns:

            """
            if not hasattr(self, "_db"):
                from core_main_app.utils.databases.mongo import MONGO_CLIENT

                self._db = MONGO_CLIENT[MONGO_DB]

            return GridFS(self._db, collection=self.collection)

        def path(self, name):
            pass

        def listdir(self, path):
            pass

        def get_accessed_time(self, name):
            pass

        def get_modified_time(self, name):
            pass



""" Custom admin site for the User Preferences model
"""

from core_main_app.utils.admin_site.model_admin_class import (
    get_base_model_admin_class,
)


class CustomUserPreferencesAdmin(
    get_base_model_admin_class("UserPreferences")
):
    """Custom User Preferences Admin"""

    list_display = [
        "user_id",
        "timezone",
    ]

    def has_add_permission(self, request, obj=None):
        """Prevent from manually adding user preferences"""
        return False



""" Authentication tests for Web Page REST API
"""
from unittest.mock import patch

from django.test import SimpleTestCase
from rest_framework import status

from core_main_app.components.web_page.models import WebPage
from core_main_app.rest.web_page import views as web_page_views
from core_main_app.rest.web_page.serializers import WebPageSerializer
from core_main_app.utils.tests_tools.MockUser import create_mock_user
from core_main_app.utils.tests_tools.RequestMock import RequestMock


class TestWebPageListGetPermission(SimpleTestCase):
    """TestWebPageListGetPermission"""

    @patch("core_main_app.components.web_page.api.get")
    def test_anonymous_returns_http_200(self, web_page_api_get):
        """test_anonymous_returns_http_200

        Args:
            web_page_api_get:

        Returns:

        """
        web_page_api_get.return_value = _get_mock_web_page()

        response = RequestMock.do_request_get(
            web_page_views.WebPageList.as_view(web_page_type="login"), None
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch("core_main_app.components.web_page.api.get")
    def test_authenticated_returns_http_200(self, web_page_api_get):
        """test_authenticated_returns_http_200

        Args:
            web_page_api_get:

        Returns:

        """
        web_page_api_get.return_value = _get_mock_web_page()
        mock_user = create_mock_user("1")

        response = RequestMock.do_request_get(
            web_page_views.WebPageList.as_view(web_page_type="login"),
            mock_user,
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch("core_main_app.components.web_page.api.get")
    def test_staff_returns_http_200(self, web_page_api_get):
        """test_staff_returns_http_200

        Args:
            web_page_api_get:

        Returns:

        """
        web_page_api_get.return_value = _get_mock_web_page()
        mock_user = create_mock_user("1", is_staff=True)

        response = RequestMock.do_request_get(
            web_page_views.WebPageList.as_view(web_page_type="login"),
            mock_user,
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestWebPageListPostPermission(SimpleTestCase):
    """TestWebPageListPostPermission"""

    def test_anonymous_returns_http_403(self):
        """test_anonymous_returns_http_403

        Returns:

        """
        response = RequestMock.do_request_post(
            web_page_views.WebPageList.as_view(web_page_type="login"), None
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_authenticated_returns_http_403(self):
        """test_authenticated_returns_http_403

        Returns:

        """
        mock_user = create_mock_user("1")

        response = RequestMock.do_request_post(
            web_page_views.WebPageList.as_view(web_page_type="login"),
            mock_user,
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch("core_main_app.components.web_page.api.get")
    @patch.object(WebPageSerializer, "is_valid")
    @patch.object(WebPageSerializer, "save")
    @patch.object(WebPageSerializer, "data")
    def test_staff_returns_http_201(
        self,
        serializer_data,
        serializer_save,
        serializer_is_valid,
        web_page_api_get,
    ):
        """test_staff_returns_http_201

        Args:
            serializer_data:
            serializer_save:
            serializer_is_valid:
            web_page_api_get:

        Returns:

        """
        web_page_api_get.return_value = _get_mock_web_page()
        serializer_data.return_value = True
        serializer_is_valid.return_value = {}
        serializer_save.return_value = None
        mock_user = create_mock_user("1", is_staff=True)

        response = RequestMock.do_request_post(
            web_page_views.WebPageList.as_view(web_page_type="login"),
            mock_user,
        )

        self.assertEqual(response.status_code, status.HTTP_201_CREATED)


class TestWebPageListDeletePermission(SimpleTestCase):
    """TestWebPageListDeletePermission"""

    def test_anonymous_returns_http_403(self):
        """test_anonymous_returns_http_403

        Returns:

        """
        response = RequestMock.do_request_delete(
            web_page_views.WebPageList.as_view(web_page_type="login"), None
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_authenticated_returns_http_403(self):
        """test_authenticated_returns_http_403

        Returns:

        """
        mock_user = create_mock_user("1")

        response = RequestMock.do_request_delete(
            web_page_views.WebPageList.as_view(web_page_type="login"),
            mock_user,
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch("core_main_app.components.web_page.api.delete_by_type")
    @patch.object(WebPageSerializer, "is_valid")
    @patch.object(WebPageSerializer, "save")
    @patch.object(WebPageSerializer, "data")
    def test_staff_returns_http_204(
        self,
        serializer_data,
        serializer_save,
        serializer_is_valid,
        web_page_api_delete_by_type,
    ):
        """test_staff_returns_http_204

        Args:
            serializer_data:
            serializer_save:
            serializer_is_valid:
            web_page_api_delete_by_type:

        Returns:

        """
        web_page_api_delete_by_type.return_value = None
        serializer_data.return_value = True
        serializer_is_valid.return_value = {}
        serializer_save.return_value = None
        mock_user = create_mock_user("1", is_staff=True)

        response = RequestMock.do_request_delete(
            web_page_views.WebPageList.as_view(web_page_type="login"),
            mock_user,
        )

        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)


def _get_mock_web_page():
    """Returns a Web page"""
    type_default_for_test = 0
    return WebPage(type=type_default_for_test, content="web page")



from unittest.mock import MagicMock, patch

from core_main_app.commons.exceptions import DoesNotExist
from core_main_app.utils.tests_tools.MockUser import create_mock_user
from django.contrib.sessions.middleware import SessionMiddleware
from django.test import SimpleTestCase, RequestFactory

from core_main_app.components.user_preferences.models import UserPreferences
from core_main_app.middleware.timezone import (
    TimezoneMiddleware,
    USER_TIMEZONE_NOT_SET,
)


class TestTimezoneMiddleware(SimpleTestCase):
    """TestTimezoneMiddleware"""

    def setUp(self):
        """setUp

        Returns:

        """
        self.factory = RequestFactory()

    @patch("core_main_app.components.user_preferences.api.get_by_user")
    @patch("django.utils.timezone.activate")
    def test_timezone_middleware_set(
        self, timezone_activate, mock_get_by_user
    ):
        """test_timezone_middleware_set

        Returns:

        """
        # Arrange
        get_response = MagicMock()
        request = self.factory.post("core_main_set_timezone")
        request.user = create_mock_user("1")
        mock_get_by_user.return_value = UserPreferences(
            user_id=request.user.id, timezone="UTC"
        )
        # Add middlewares
        mock_get_response = MagicMock()
        middleware = SessionMiddleware(mock_get_response)
        middleware.process_request(request)

        # Act
        middleware = TimezoneMiddleware(get_response)
        middleware(request)

        # Assert
        self.assertTrue(timezone_activate.called)

    @patch("core_main_app.components.user_preferences.api.get_by_user")
    @patch("django.utils.timezone.deactivate")
    def test_timezone_middleware_not_set(
        self, timezone_deactivate, mock_get_by_user
    ):
        """test_timezone_middleware_not_set

        Returns:

        """
        # Arrange
        get_response = MagicMock()
        request = self.factory.post("core_main_set_timezone")
        request.user = create_mock_user("1")
        mock_get_by_user.return_value = UserPreferences(
            user_id=request.user.id,
        )
        # Add middlewares
        mock_get_response = MagicMock()
        middleware = SessionMiddleware(mock_get_response)
        middleware.process_request(request)

        # Act
        middleware = TimezoneMiddleware(get_response)
        middleware(request)

        # Assert
        self.assertTrue(timezone_deactivate.called)

    @patch("core_main_app.components.user_preferences.api.get_by_user")
    @patch("django.utils.timezone.deactivate")
    def test_timezone_middleware_user_preferences_not_found(
        self, timezone_deactivate, mock_get_by_user
    ):
        """test_timezone_middleware_not_set

        Returns:

        """
        # Arrange
        get_response = MagicMock()
        request = self.factory.post("core_main_set_timezone")
        request.user = create_mock_user("1")
        mock_get_by_user.side_effect = DoesNotExist("error")

        # Add middlewares
        mock_get_response = MagicMock()
        middleware = SessionMiddleware(mock_get_response)
        middleware.process_request(request)

        # Act
        middleware = TimezoneMiddleware(get_response)
        middleware(request)

        # Assert
        self.assertTrue(timezone_deactivate.called)
        self.assertEqual(
            request.session["django_timezone"], USER_TIMEZONE_NOT_SET
        )



""" User Preferences model
"""

from django.core.exceptions import ObjectDoesNotExist, ValidationError
from django.db import models
from pytz import common_timezones as pytz_common_timezones

from core_main_app.commons import exceptions


def validate_timezone(timezone):
    """
    validate_timezone
    """
    if timezone not in pytz_common_timezones:
        raise ValidationError("Invalid timezone value.")


class UserPreferences(models.Model):
    """User Preferences"""

    user_id = models.CharField(
        blank=False, max_length=200, null=False, unique=True
    )

    timezone = models.CharField(
        blank=True, null=True, max_length=200, validators=[validate_timezone]
    )

    class Meta:
        verbose_name = " User Preferences"
        verbose_name_plural = " User Preferences"

    @staticmethod
    def get_by_user(user):
        """Get user preferences relative to the given id

        Args:
            user:

        Returns:

        """
        try:
            return UserPreferences.objects.get(user_id=str(user.id))
        except ObjectDoesNotExist as exception:
            raise exceptions.DoesNotExist(str(exception))
        except Exception as exception:
            raise exceptions.ModelError(str(exception))

    def save_object(self):
        """Custom save

        Returns:

        """
        self.full_clean()
        return self.save()



"""Mongo query builder tools
"""
import copy
import re

from django.conf import settings

from core_main_app.commons.exceptions import QueryError
from core_main_app.utils.databases.backend import uses_postgresql_backend


def _compile_regex(query):
    """Compile all regular expressions in the query

    Args:
        query:

    Returns:

    """
    for key, value in query.items():
        if key == "$and" or key == "$or":
            for sub_value in value:
                _compile_regex(sub_value)
        elif isinstance(value, str) or isinstance(value, str):
            if len(value) >= 2 and value[0] == "/" and value[-1] == "/":
                query[key] = re.compile(value[1:-1])
        elif isinstance(value, dict):
            _compile_regex(value)


def _add_sub_document_root(query, sub_document_root):
    """Adds a sub document root to each criteria

    Returns:

    """
    for key in list(query.keys()):
        if key == "$and" or key == "$or":
            for value in query[key]:
                _add_sub_document_root(value, sub_document_root)
        elif not key.startswith("$"):
            query[f"{sub_document_root}.{key}"] = query.pop(key)


def prepare_query(query_dict, regex=True, sub_document_root=None):
    """Prepares the query to before executing it

    Args:
        query_dict:
        regex:
        sub_document_root:

    Returns:

    """
    # get a copy of the query
    query = copy.deepcopy(query_dict)

    if regex:
        # compile the regular expressions
        _compile_regex(query)

    if sub_document_root is not None:
        # add a sub document root
        _add_sub_document_root(query, sub_document_root)

    return query


def get_access_filters_from_query(
    query_dict, workspace_list=None, user_list=None
):
    """Get workspaces criteria from queries

    Args:
        query_dict:
        workspace_list:
        user_list:

    Returns:

    """
    # create empty list of workspaces if not defined
    workspace_list = [] if not workspace_list else workspace_list
    # create empty list of users if not defined
    user_list = [] if not user_list else user_list
    # iterate through query dict
    for key, value in query_dict.items():
        # if key is workspace
        if key == "workspace":
            # if filter on a list of workspaces
            if (
                isinstance(query_dict["workspace"], dict)
                and "$in" in query_dict["workspace"]
            ):
                # append each workspace id to list of workspaces
                for workspace_id in query_dict["workspace"]["$in"]:
                    workspace_list.append(workspace_id)
            # if filter on a single value
            elif (
                isinstance(query_dict["workspace"], int)
                or isinstance(query_dict["workspace"], str)
                or query_dict["workspace"] is None
            ):
                # add workspace id to the list of workspaces
                workspace_list.append(query_dict["workspace"])
        # if key is user_id
        elif key == "user_id":
            # if filter on a list of user ids
            if (
                isinstance(query_dict["user_id"], dict)
                and "$in" in query_dict["user_id"]
            ):
                # append each user id to list of user ids
                for user_id in query_dict["user_id"]["$in"]:
                    user_list.append(str(user_id))
            # if filter on a single value
            elif isinstance(query_dict["user_id"], int) or isinstance(
                query_dict["user_id"], str
            ):
                # add user id to the list of users
                user_list.append(str(query_dict["user_id"]))
        else:
            # if AND or OR operators found
            if key in ["$and", "$or"]:
                # iterate though sub dict
                for sub_value in value:
                    (
                        sub_workspace_list,
                        sub_user_list,
                    ) = get_access_filters_from_query(
                        sub_value, workspace_list, user_list
                    )
                    # add values found in sub dict to existing lists
                    if sub_workspace_list:
                        workspace_list.extend(sub_workspace_list)
                    if sub_user_list:
                        user_list.extend(sub_user_list)
    # return list of workspaces and users
    return workspace_list, user_list


def convert_to_django(query_dict):
    """Extract criteria from mongodb query and translates them to django ORM

    Returns:

    """
    if settings.MONGODB_INDEXING:
        from mongoengine.queryset.visitor import Q

        template_key = "_template_id"
    else:
        from django.db.models import Q

        template_key = "template"

    # check $where operator
    if "$where" in str(query_dict):
        raise QueryError("Unsupported operator found")

    # create a query object
    q_list = Q()
    # iterate through query dict key/value pairs
    for key, value in query_dict.items():
        # if the key is not an operator
        if not key.startswith("$"):
            # check if $ found in key
            if "$" in key:
                raise QueryError("Unsupported $ operator found")
            # ignore workspace and user_id filters (dealt with by acl layer)
            if key in ["workspace", "_workspace_id", "user_id", "_id", "id"]:
                continue
            # if key is template
            elif key == "template":
                # check if filtering by a list
                if (
                    isinstance(query_dict["template"], dict)
                    and "$in" in query_dict["template"]
                ):
                    # add filter by list of templates
                    q_list &= Q(
                        **{
                            f"{template_key}__in": sanitize_value(
                                query_dict["template"]["$in"]
                            )
                        }
                    )
                # check if filtering by single value
                elif isinstance(query_dict["template"], int) or isinstance(
                    query_dict["template"], str
                ):
                    # add filter by value (template id)
                    q_list &= Q(
                        **{
                            template_key: sanitize_value(
                                query_dict["template"]
                            )
                        }
                    )
            else:
                # initialize negate var to invert django queries
                negate = False
                # replace dots by double underscores (django notation)
                key = key.replace(".", "__") if "." in key else key
                # initialize operator
                operator = "exact"
                # check if not operator
                if isinstance(value, dict) and "$not" in value:
                    negate = True
                    # add not to key
                    key = (
                        f"{key}__not"
                        if negate and settings.MONGODB_INDEXING
                        else key
                    )
                    # move value to document in $not
                    value = value["$not"]
                # if value is a regex
                if isinstance(value, re.Pattern):
                    # set regex operator
                    operator = "regex"
                    # set value with regex pattern
                    value = value.pattern
                # if value is None
                elif value is None:
                    # mongo: __exact = None / django: __exact = None
                    value = None
                # if the value is a dict
                elif isinstance(value, dict):
                    # check if ne operator (not equal)
                    if "$ne" in value:
                        # set not equal to create query not
                        negate = True
                        # set ne operator
                        operator = (
                            "ne" if settings.MONGODB_INDEXING else operator
                        )
                        # set value
                        value = value["$ne"]
                    # check if eq operator (equal to)
                    elif "$eq" in value:
                        # set value
                        value = value["$eq"]
                    # check if lt operator (less than)
                    elif "$lt" in value:
                        # set lt operator
                        operator = "lt"
                        # set value
                        value = sanitize_number(value["$lt"])
                    # check if lt operator (less than or equal)
                    elif "$lte" in value:
                        # set lte operator
                        operator = "lte"
                        # set value
                        value = sanitize_number(value["$lte"])
                    # check if gt operator (greater than)
                    elif "$gt" in value:
                        # set gt operator
                        operator = "gt"
                        # set value
                        value = sanitize_number(value["$gt"])
                    # check if gte operator (greater than or equal)
                    elif "$gte" in value:
                        # set gte operator
                        operator = "gte"
                        # set value
                        value = sanitize_number(value["$gte"])
                    # check if in operator (included in list)
                    elif "$in" in value:
                        # set in operator
                        operator = "in"
                        # set value
                        value = value["$in"]
                    # check if regex operator
                    elif "$regex" in value:
                        # set regex operator
                        operator = "regex"
                        # set the value
                        value = value["$regex"]
                    # check if exists operator
                    elif "$exists" in value:
                        # skip case where set to False for now (i.e. can not look for documents where path is absent)
                        if value["$exists"] is False:
                            raise QueryError(
                                'Unsupported operator found: {"$exists": False}'
                            )
                        if settings.MONGODB_INDEXING:
                            # set exists operator
                            operator = "exists"
                            value = True
                        else:
                            # value to look for is the last element of the key
                            # (e.g. if key is dict_content__root__element, value becomes element)
                            value = key.split("__")[-1]
                            # key is the rest of the path, minus the last part that was just set as value
                            # (e.g. dict_content__root__element, key is dict_content__root)
                            key = "__".join(key.split("__")[:-1])
                            # add has key operator to key (e.g. dict_content__root, becomes dict_content__root__has_key)
                            operator = "has_key"
                    else:
                        # If an operator not listed above is found, an exception is raised
                        raise QueryError(
                            f"Unsupported operator found: {value}"
                        )
                # build query
                query = Q(**{f"{key}__{operator}": sanitize_value(value)})
                # invert django query if negate is true
                query = (
                    ~query
                    if negate and not settings.MONGODB_INDEXING
                    else query
                )

                # add AND query filter from string key and value
                q_list &= query
        else:
            # if operator and
            if key == "$and":
                # iterate though sub dict
                for sub_value in value:
                    # add AND filters to the query
                    q_list &= convert_to_django(sub_value)
            # if operator or
            elif key == "$or":
                # iterate through sub dict
                for sub_value in value:
                    # add OR filters to the query
                    q_list |= convert_to_django(sub_value)
            # if operators text and search found
            elif key == "$text" and "$search" in value:
                # get text query
                text_query = query_dict["$text"]["$search"]
                # check text query is a string
                if not isinstance(text_query, str):
                    raise QueryError(f"Unsupported value found in: {key}")
                # strip white spaces
                text_query = text_query.strip()
                # sanitize string
                sanitize_value(text_query)
                # if data stored in MongoDB
                if settings.MONGODB_INDEXING:
                    q_list &= Q(__raw__={"$text": {"$search": text_query}})
                # if data stored in PostgreSQL
                elif uses_postgresql_backend():
                    q_list &= Q(vector_column=text_query)
                else:
                    # extract keywords from dict
                    for keyword in text_query.split(" "):
                        # add text filter
                        q_list &= Q(
                            dict_content__icontains=keyword.replace('"', "")
                        )
            else:
                # raise an error if another operator was found
                raise QueryError(f"Unsupported operator found: {key}")

    return q_list


def sanitize_number(value):
    """Sanitize number


    Args:
        value:

    Returns:

    """
    # check type of value
    if not (isinstance(value, (int, float))):
        raise QueryError("Unsupported value: expected a number.")
    return value


def sanitize_value(value):
    """Sanitize value

    Args:
        value:

    Returns:

    """
    # check if operator in value
    if "$" in str(value):
        raise QueryError("Unsupported $ operator found.")
    # if value is a list
    if isinstance(value, list):
        # check for $ sign in the list
        for item in value:
            sanitize_value(item)
    return value



"""Custom context processor
"""
from django.conf import settings
from core_main_app.commons.constants import AVAILABLE_BOOTSTRAP_VERSIONS


def domain_context_processor(request):
    """domain_context_processor

    Args:
        request:

    Returns
    """

    return {
        "WEBSITE_ADMIN_COLOR": settings.WEBSITE_ADMIN_COLOR
        if hasattr(settings, "WEBSITE_ADMIN_COLOR")
        else "black",
        "WEBSITE_SHORT_TITLE": settings.WEBSITE_SHORT_TITLE
        if hasattr(settings, "WEBSITE_SHORT_TITLE")
        else "",
        "CUSTOM_TITLE": settings.CUSTOM_TITLE
        if hasattr(settings, "CUSTOM_TITLE")
        else "",
        "CUSTOM_ORGANIZATION": settings.CUSTOM_ORGANIZATION
        if hasattr(settings, "CUSTOM_ORGANIZATION")
        else "",
        "CUSTOM_NAME": settings.CUSTOM_NAME,
        "CUSTOM_SUBTITLE": settings.CUSTOM_SUBTITLE
        if hasattr(settings, "CUSTOM_SUBTITLE")
        else "",
        "CUSTOM_DATA": settings.CUSTOM_DATA
        if hasattr(settings, "CUSTOM_DATA")
        else "",
        "CUSTOM_URL": settings.CUSTOM_URL
        if hasattr(settings, "CUSTOM_URL")
        else "",
        "DATA_SOURCES_EXPLORE_APPS": settings.DATA_SOURCES_EXPLORE_APPS
        if hasattr(settings, "DATA_SOURCES_EXPLORE_APPS")
        else [],
        "INSTALLED_APPS": settings.INSTALLED_APPS
        if hasattr(settings, "INSTALLED_APPS")
        else [],
        "DISPLAY_NIST_HEADERS": settings.DISPLAY_NIST_HEADERS
        if hasattr(settings, "DISPLAY_NIST_HEADERS")
        else False,
        "GA_TRACKING_ID": settings.GA_TRACKING_ID
        if hasattr(settings, "GA_TRACKING_ID")
        else None,
        "LOGIN_URL": settings.LOGIN_URL
        if hasattr(settings, "LOGIN_URL")
        else "",
        "BOOTSTRAP_VERSION": settings.BOOTSTRAP_VERSION
        if hasattr(settings, "BOOTSTRAP_VERSION")
        and settings.BOOTSTRAP_VERSION in AVAILABLE_BOOTSTRAP_VERSIONS
        else "4.6.2",
        "PROJECT_VERSION": settings.PROJECT_VERSION
        if hasattr(settings, "PROJECT_VERSION")
        else "0.0.0",
    }



""" Fixtures file for template version manager
"""
from core_main_app.components.template.models import Template
from core_main_app.components.template_version_manager.models import (
    TemplateVersionManager,
)
from core_main_app.utils.integration_tests.fixture_interface import (
    FixtureInterface,
)


class TemplateVersionManagerFixtures(FixtureInterface):
    """Template Version Manager fixtures"""

    template_1_1 = None
    template_1_2 = None
    template_1_3 = None
    template_2_1 = None
    template_vm_1 = None
    template_vm_2 = None
    template_vm_collection = None

    def insert_data(self):
        """Insert a set of Templates and Template Version Managers.

        Returns:

        """
        self.template_vm_1 = TemplateVersionManager(
            title="template 1",
            user=None,
            is_disabled=False,
        )
        self.template_vm_1.save_version_manager()
        self.template_1_1 = Template(
            filename="template1_1.xsd",
            content="content1_1",
            hash="hash1_1",
            version_manager=self.template_vm_1,
        )
        self.template_1_1.save()
        self.template_1_2 = Template(
            filename="template1_2.xsd",
            content="content1_2",
            hash="hash1_2",
            is_disabled=True,
            version_manager=self.template_vm_1,
        )
        self.template_1_2.save()
        self.template_1_3 = Template(
            filename="template1_3.xsd",
            content="content1_3",
            hash="hash1_3",
            is_current=True,
            version_manager=self.template_vm_1,
        )
        self.template_1_3.save()

        self.template_vm_2 = TemplateVersionManager(
            title="template 2",
            user="1",
            is_disabled=False,
        )
        self.template_vm_2.save_version_manager()
        self.template_2_1 = Template(
            filename="template2_1.xsd",
            content="content2_1",
            hash="hash2_1",
            is_current=True,
            version_manager=self.template_vm_2,
        )
        self.template_2_1.save()

        self.template_vm_collection = [self.template_vm_1, self.template_vm_2]


class TemplateVersionManagerAccessControlFixtures(FixtureInterface):
    """Template Version Manager fixtures"""

    user1_template = None
    user2_template = None
    global_template = None
    user1_tvm = None
    user2_tvm = None
    global_tvm = None
    template_vm_collection = None

    def insert_data(self):
        """Insert a set of Templates and Template Version Managers.

        Returns:

        """
        # Make a connexion with a mock database
        xsd = (
            '<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">'
            '<xs:element name="tag"></xs:element></xs:schema>'
        )

        self.user1_tvm = TemplateVersionManager(
            title="template 1",
            user="1",
            is_disabled=False,
        )
        self.user1_tvm.save_version_manager()

        self.user2_tvm = TemplateVersionManager(
            title="template 2",
            user="2",
            is_disabled=False,
        )
        self.user2_tvm.save_version_manager()

        self.global_tvm = TemplateVersionManager(
            title="global template",
            user=None,
            is_disabled=False,
        )
        self.global_tvm.save_version_manager()

        self.user1_template = Template(
            filename="template1.xsd",
            content=xsd,
            hash="hash1",
            user="1",
            is_current=True,
            version_manager=self.user1_tvm,
        )
        self.user1_template.save()
        self.user2_template = Template(
            filename="template2.xsd",
            content=xsd,
            hash="hash2",
            user="2",
            is_current=True,
            version_manager=self.user2_tvm,
        )
        self.user2_template.save()
        self.global_template = Template(
            filename="global_template.xsd",
            content=xsd,
            hash="global hash",
            user=None,
            is_current=True,
            version_manager=self.global_tvm,
        )
        self.global_template.save()

        self.template_vm_collection = [
            self.user1_tvm,
            self.user2_tvm,
            self.global_tvm,
        ]


class TemplateVersionManagerOrderingFixtures(FixtureInterface):
    """Template Version Manager Ordering fixtures"""

    user1_template = None
    global_template = None
    tvm1 = None
    tvm2 = None
    global_tvm1 = None
    global_tvm2 = None
    template_vm_collection = None

    def insert_data(self):
        """Insert a set of Templates and Template Version Managers.

        Returns:

        """
        # Make a connexion with a mock database
        xsd = (
            '<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">'
            '<xs:element name="tag"></xs:element></xs:schema>'
        )

        self.tvm1 = TemplateVersionManager(
            title="template 1",
            user="1",
            is_disabled=False,
        )
        self.tvm1.save_version_manager()

        self.tvm2 = TemplateVersionManager(
            title="template 2",
            user="1",
            is_disabled=False,
        )
        self.tvm2.save_version_manager()

        self.global_tvm1 = TemplateVersionManager(
            title="global template1",
            user=None,
            is_disabled=False,
        )
        self.global_tvm1.save_version_manager()

        self.global_tvm2 = TemplateVersionManager(
            title="global template2",
            user=None,
            is_disabled=False,
        )
        self.global_tvm2.save_version_manager()

        self.user1_template = Template(
            filename="template1.xsd",
            content=xsd,
            hash="hash1",
            user="1",
            is_current=True,
            version_manager=self.tvm1,
        )
        self.user1_template.save()

        self.global_template = Template(
            filename="global_template.xsd",
            content=xsd,
            hash="global hash",
            user=None,
            is_current=True,
            version_manager=self.global_tvm1,
        )
        self.global_template.save()

        self.template_vm_collection = [
            self.tvm1,
            self.tvm2,
            self.global_tvm1,
            self.global_tvm2,
        ]



"""Integration tests for xsl transformation rest api
"""

from rest_framework import status
from tests.rest.xsl_transformation.fixtures.fixtures import (
    XslTransformationFixtures,
)

from core_main_app.rest.xsl_transformation import views as xsl_views
from core_main_app.utils.integration_tests.integration_base_test_case import (
    IntegrationBaseTestCase,
)
from core_main_app.utils.tests_tools.MockUser import create_mock_user
from core_main_app.utils.tests_tools.RequestMock import RequestMock

fixture_data = XslTransformationFixtures()


class TestGetAllXslTransformationList(IntegrationBaseTestCase):
    """TestGetAllXslTransformationList"""

    fixture = fixture_data

    def setUp(self):
        """setUp

        Returns:

        """
        super().setUp()

    def test_get_all_returns_status_403_with_no_permission_needed(self):
        """test_get_all_returns_status_403_with_no_permission_needed

        Returns:

        """
        # Arrange
        user = create_mock_user("1")

        # Act
        response = RequestMock.do_request_get(
            xsl_views.XslTransformationList.as_view(), user
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)


class TestPostXslTransformationList(IntegrationBaseTestCase):
    """TestPostXslTransformationList"""

    fixture = fixture_data

    def setUp(self):
        """setUp

        Returns:

        """
        super().setUp()
        self.data = None

    def test_post_returns_status_403_if_user_is_unauthorized(self):
        """test_post_returns_status_403_if_user_is_unauthorized

        Returns:

        """
        # Arrange
        user = create_mock_user("0")

        # Act
        response = RequestMock.do_request_post(
            xsl_views.XslTransformationList.as_view(), user, self.data
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_post_returns_status_400_if_data_are_not_valid_with_admin_user(
        self,
    ):
        """test_post_returns_status_400_if_data_are_not_valid_with_admin_user

        Returns:

        """
        # Arrange
        user = create_mock_user("0", True, True)

        # Act
        response = RequestMock.do_request_post(
            xsl_views.XslTransformationList.as_view(), user, self.data
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)

    def test_post_returns_status_201_if_data_are_valid_with_admin_user(self):
        """test_post_returns_status_201_if_data_are_valid_with_admin_user

        Returns:

        """
        # Arrange
        user = create_mock_user("0", True, True)
        self.data = {
            "name": "name",
            "filename": "filename.xsd",
            "content": '<?xml version="1.0" encoding="UTF-8"?>'
            '<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">'
            "<xsl:template></xsl:template></xsl:stylesheet>",
        }

        # Act
        response = RequestMock.do_request_post(
            xsl_views.XslTransformationList.as_view(), user, self.data
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)


class TestGetXslTransformationDetail(IntegrationBaseTestCase):
    """TestGetXslTransformationDetail"""

    fixture = fixture_data

    def setUp(self):
        """setUp

        Returns:

        """
        super().setUp()
        self.data = None

    def test_get_returns_object_when_found(self):
        """test_get_returns_object_when_found

        Returns:

        """
        # Arrange
        user = create_mock_user("0", True)
        self.param = {"pk": self.fixture.data_1.id}

        # Act
        response = RequestMock.do_request_get(
            xsl_views.XslTransformationDetail.as_view(),
            user,
            self.data,
            self.param,
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    def test_get_raise_404_when_not_found(self):
        """test_get_raise_404_when_not_found

        Returns:

        """
        # Arrange
        user = create_mock_user("0", True)
        self.param = {"pk": -1}

        # Act
        response = RequestMock.do_request_get(
            xsl_views.XslTransformationDetail.as_view(),
            user,
            self.data,
            self.param,
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    def test_get_raise_500_sever_error_when_general_error_occurred(self):
        """test_get_raise_500_sever_error_when_general_error_occurred

        Returns:

        """
        # Arrange
        user = create_mock_user("0", True)
        self.param = {"pk": "test"}

        # Act
        response = RequestMock.do_request_get(
            xsl_views.XslTransformationDetail.as_view(),
            user,
            self.data,
            self.param,
        )

        # Assert
        self.assertEqual(
            response.status_code, status.HTTP_500_INTERNAL_SERVER_ERROR
        )


class TestDeleteXslTransformationDetail(IntegrationBaseTestCase):
    """TestDeleteXslTransformationDetail"""

    fixture = fixture_data

    def setUp(self):
        """setUp

        Returns:

        """
        super().setUp()
        self.data = None

    def test_delete_raise_403_if_user_is_unauthorized(self):
        """test_delete_raise_403_if_user_is_unauthorized

        Returns:

        """
        # Arrange
        user = create_mock_user("0")
        self.param = {"pk": 1}

        # Act
        response = RequestMock.do_request_delete(
            xsl_views.XslTransformationDetail.as_view(),
            user,
            self.data,
            self.param,
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_delete_raise_404_when_not_found(self):
        """test_delete_raise_404_when_not_found

        Returns:

        """
        # Arrange
        user = create_mock_user("0", True, True)
        self.param = {"pk": -1}

        # Act
        response = RequestMock.do_request_delete(
            xsl_views.XslTransformationDetail.as_view(),
            user,
            self.data,
            self.param,
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    def test_post_raise_500_sever_error_when_general_error_occurred(self):
        """test_post_raise_500_sever_error_when_general_error_occurred

        Returns:

        """
        # Arrange
        user = create_mock_user("0", True, True)
        self.param = {"pk": "test"}

        # Act
        response = RequestMock.do_request_delete(
            xsl_views.XslTransformationDetail.as_view(),
            user,
            self.data,
            self.param,
        )

        # Assert
        self.assertEqual(
            response.status_code, status.HTTP_500_INTERNAL_SERVER_ERROR
        )

    def test_post_return_204_if_document_is_deleted_whit_success(self):
        """test_post_return_204_if_document_is_deleted_whit_success

        Returns:

        """
        # Arrange
        user = create_mock_user("0", True, True)
        self.param = {"pk": self.fixture.data_1.id}

        # Act
        response = RequestMock.do_request_delete(
            xsl_views.XslTransformationDetail.as_view(),
            user,
            self.data,
            self.param,
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)


class TestPatchXslTransformationDetail(IntegrationBaseTestCase):
    """TestPatchXslTransformationDetail"""

    fixture = fixture_data

    def setUp(self):
        """setUp

        Returns:

        """
        super().setUp()
        self.data = None

    def test_patch_raise_403_if_user_is_authorized(self):
        """test_patch_raise_403_if_user_is_authorized

        Returns:

        """
        # Arrange
        user = create_mock_user("0")
        self.param = {"pk": 1}

        # Act
        response = RequestMock.do_request_patch(
            xsl_views.XslTransformationDetail.as_view(),
            user,
            self.data,
            self.param,
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_patch_raise_404_when_not_found(self):
        """test_patch_raise_404_when_not_found

        Returns:

        """
        # Arrange
        user = create_mock_user("0", True, True)
        self.param = {"pk": -1}

        # Act
        response = RequestMock.do_request_patch(
            xsl_views.XslTransformationDetail.as_view(),
            user,
            self.data,
            self.param,
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    def test_patch_raise_400_sever_error_when_general_error_occurred(self):
        """test_patch_raise_400_sever_error_when_general_error_occurred

        Returns:

        """
        # Arrange
        user = create_mock_user("0", True, True)
        self.param = {"pk": self.fixture.data_1.id}

        # Act
        response = RequestMock.do_request_patch(
            xsl_views.XslTransformationDetail.as_view(), user, param=self.param
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)

    def test_patch_returns_200_when_data_are_valid_with_authorized_user(self):
        """test_patch_returns_200_when_data_are_valid_with_authorized_user

        Returns:

        """
        # Arrange
        user = create_mock_user("0", True, True)
        self.param = {"pk": self.fixture.data_1.id}

        self.data = {
            "name": "test_",
            "filename": "test_filename_",
        }

        # Act
        response = RequestMock.do_request_patch(
            xsl_views.XslTransformationDetail.as_view(),
            user,
            self.data,
            self.param,
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestPostXslTransformationTransform(IntegrationBaseTestCase):
    """TestPostXslTransformationTransform"""

    fixture = fixture_data

    def setUp(self):
        """setUp

        Returns:

        """
        super().setUp()
        self.data = None

    def test_post_raise_error_500_if_xslt_does_not_exist(self):
        """test_post_raise_error_500_if_xslt_does_not_exist

        Returns:

        """
        # Arrange
        user = create_mock_user("0")

        self.data = {
            "xml_content": "xml_content",
            "xslt_name": "xslt_name",
        }

        # Act
        response = RequestMock.do_request_post(
            xsl_views.XslTransformationTransform.as_view(), user, self.data
        )

        # Assert
        self.assertEqual(
            response.status_code, status.HTTP_500_INTERNAL_SERVER_ERROR
        )

    def test_post_return_status_200_if_xml_and_xslt_name_are_valid_parameters(
        self,
    ):
        """test_post_return_status_200_if_xml_and_xslt_name_are_valid_parameters

        Returns:

        """
        # Arrange
        user = create_mock_user("0")

        self.data = {
            "xml_content": "<test></test>",
            "xslt_name": "name_1",
        }

        # Act
        response = RequestMock.do_request_post(
            xsl_views.XslTransformationTransform.as_view(), user, self.data
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)



""" Int test Blob
"""

from django.core.files.uploadedfile import SimpleUploadedFile
from tests.components.blob.fixtures.fixtures import (
    BlobFixtures,
    AccessControlBlobFixture,
)

from core_main_app.commons import exceptions
from core_main_app.components.blob.models import Blob
from core_main_app.utils.integration_tests.integration_base_test_case import (
    IntegrationBaseTestCase,
)
from core_main_app.utils.tests_tools.MockUser import create_mock_user
from core_main_app.components.blob import api as blob_api

fixture_blob = BlobFixtures()
fixture_blob_workspace = AccessControlBlobFixture()


class TestBlobGetAll(IntegrationBaseTestCase):
    """TestBlobGetAll"""

    fixture = fixture_blob

    def test_blob_get_all_return_collection_of_blob(self):
        """test_blob_get_all_return_collection_of_blob

        Returns:

        """
        # Act
        result = Blob.get_all()
        # Assert
        self.assertTrue(all(isinstance(item, Blob) for item in result))

    def test_blob_get_all_return_objects_blob_in_collection(self):
        """test_blob_get_all_return_objects_blob_in_collection

        Returns:

        """
        # Act
        result = Blob.get_all()
        # Assert
        self.assertTrue(len(self.fixture.blob_collection) == result.count())


class TestBlobGetById(IntegrationBaseTestCase):
    """TestBlobGetById"""

    fixture = fixture_blob

    def test_blob_get_by_id_raises_does_not_exist_error_if_not_found(self):
        """test_blob_get_by_id_raises_does_not_exist_error_if_not_found

        Returns:

        """
        # Act # Assert
        with self.assertRaises(exceptions.DoesNotExist):
            Blob.get_by_id(-1)

    def test_blob_get_by_id_return_blob_if_found(self):
        """test_blob_get_by_id_return_blob_if_found

        Returns:

        """
        # Act
        result = Blob.get_by_id(self.fixture.blob_1.id)
        # Assert
        self.assertEqual(result, self.fixture.blob_1)


class TestBlobGetAllByUserId(IntegrationBaseTestCase):
    """TestBlobGetAllByUserId"""

    fixture = fixture_blob

    def test_blob_get_all_by_user_id_return_collection_of_blob_from_user(self):
        """test_blob_get_all_by_user_id_return_collection_of_blob_from_user

        Returns:

        """
        # Arrange
        user_id = 1
        # Act
        result = Blob.get_all_by_user_id(user_id)
        # Assert
        self.assertTrue(all(item.user_id == str(user_id) for item in result))

    def test_blob_get_all_by_user_id_return_empty_collection_of_blob_from_user_with_no_associated_blob(
        self,
    ):
        """test_blob_get_all_by_user_id_return_empty_collection_of_blob_from_user_with_no_associated_blob

        Returns:

        """
        # Arrange
        user_id = 800
        # Act
        result = Blob.get_all_by_user_id(user_id)
        # Assert
        self.assertTrue(result.count() == 0)


class TestBlobGetAllByWorkspace(IntegrationBaseTestCase):
    """TestBlobGetAllByWorkspace"""

    fixture = fixture_blob_workspace

    def test_blob_get_all_by_workspace_return_collection_of_blob_from_user(
        self,
    ):
        """test_blob_get_all_by_workspace_return_collection_of_blob_from_user

        Returns:

        """
        # Act
        result = Blob.get_all_by_workspace(self.fixture.workspace_1)
        # Assert
        self.assertTrue(
            all(
                item.user_id == str(self.fixture.workspace_1.owner)
                for item in result
            )
        )

    def test_blob_get_all_by_workspace_return_empty_collection_of_blob_from_empty_workspace(
        self,
    ):
        """test_blob_get_all_by_workspace_return_empty_collection_of_blob_from_empty_workspace

        Returns:

        """
        # Act
        result = Blob.get_all_by_workspace(self.fixture.workspace_without_data)
        # Assert
        self.assertTrue(result.count() == 0)


class TestBlobGetAllByListWorkspace(IntegrationBaseTestCase):
    """TestBlobGetAllByListWorkspace"""

    fixture = fixture_blob_workspace

    def test_blob_get_all_by_workspace_return_collection_of_blob_from_user(
        self,
    ):
        """test_blob_get_all_by_workspace_return_collection_of_blob_from_user

        Returns:

        """
        # Act
        result = Blob.get_all_by_list_workspace(
            [self.fixture.workspace_1, self.fixture.workspace_2]
        )
        # Assert
        self.assertTrue(
            all(
                item.user_id == str(self.fixture.workspace_1.owner)
                or item.user_id == str(self.fixture.workspace_2.owner)
                for item in result
            )
        )

    def test_blob_get_all_by_workspace_return_empty_collection_of_blob_from_empty_workspace(
        self,
    ):
        """test_blob_get_all_by_workspace_return_empty_collection_of_blob_from_empty_workspace

        Returns:

        """
        # Act
        result = Blob.get_all_by_list_workspace(
            [self.fixture.workspace_without_data]
        )
        # Assert
        self.assertTrue(result.count() == 0)


class TestBlobInsert(IntegrationBaseTestCase):
    """TestBlobInsert"""

    def setUp(self):
        """setUp

        Returns:

        """
        self.user = create_mock_user(1, False)
        self.blob = Blob(
            filename="blob",
            user_id="1",
            blob=SimpleUploadedFile("blob.txt", b"blob"),
        )

    def test_insert_blob_creates_blob(
        self,
    ):
        """test_insert_blob_creates_blob

        Returns:

        """
        # Act
        result = blob_api.insert(self.blob, self.user)
        # Assert
        self.assertIsInstance(result, Blob)

    def test_insert_blob_raises_api_error_if_already_exists(
        self,
    ):
        """test_insert_blob_raises_api_error_if_already_exists

        Returns:

        """
        # Arrange
        blob = blob_api.insert(self.blob, self.user)
        # Act # Assert
        with self.assertRaises(exceptions.ApiError):
            blob_api.insert(blob, self.user)

    def test_insert_blob_without_blob_raises_error(self):
        """test_insert_blob_without_blob_raises_error

        Returns:

        """
        # Arrange
        blob = Blob(filename="blob", user_id="1", blob=None)

        # Act # Assert
        with self.assertRaises(exceptions.ModelError):
            blob_api.insert(blob, self.user)



""" Template access control
"""
from django.db.models import Q

from core_main_app.access_control.exceptions import AccessControlError
from core_main_app.components.template.models import Template
from django.conf import settings
from core_main_app.utils.requests_utils.access_control import (
    get_request_from_args,
)


def check_can_read_template(template, user):
    if user.is_superuser:  # No checks for superuser
        return

    # Anonymous user
    if user.is_anonymous:
        if template.user:
            raise AccessControlError(
                "Template: The user doesn't have enough rights."
            )
        if not settings.CAN_ANONYMOUS_ACCESS_PUBLIC_DOCUMENT:
            raise AccessControlError(
                "Template: The user doesn't have enough rights."
            )

    # Registered user
    if template.user and template.user != str(user.id):
        raise AccessControlError(
            "Template: The user doesn't have enough rights."
        )


def can_read_id(func, template_id, request):
    """Can read document.

    Args:
        func:
        template_id:
        request:

    Returns:

    """
    # Retrieve request and document from args
    request = get_request_from_args(template_id, request=request)
    template = func(template_id, request=request)

    # Verify that user can read the template and return the object
    check_can_read_template(template, request.user)
    return template


def can_read_global(func, *args, **kwargs):
    """Can read global template

    Args:
        func:
        *args:
        **kwargs:

    Returns:

    """
    request = get_request_from_args(*args, **kwargs)
    if request.user.is_anonymous:
        if not settings.CAN_ANONYMOUS_ACCESS_PUBLIC_DOCUMENT:
            raise AccessControlError(
                "Template: The user doesn't have enough rights."
            )
    return func(*args, **kwargs)


def can_write(func, *args, **kwargs):
    """Can write template.

    Args:
        func:
        args:
        kwargs:

    Returns:

    """

    template = next((arg for arg in args if isinstance(arg, Template)), None)

    check_can_write(template, *args, **kwargs)

    return func(*args, **kwargs)


def get_accessible_owners(request):
    """Get a list of templates owners, the user can read from

    Args:
        request:

    Returns:

    """
    if not request or request.user.is_anonymous:
        if settings.CAN_ANONYMOUS_ACCESS_PUBLIC_DOCUMENT:
            # global templates only
            return Q(user__isnull=True)
        # nothing
        return Q(user__in=[])
    if request.user.is_superuser:
        # no restrictions
        return None
    # global and owned templates
    in_q_list = Q()
    in_q_list |= Q(user__isnull=True)
    in_q_list |= Q(user=str(request.user.id))
    return in_q_list


def can_read_list(func, *args, **kwargs):
    """Can read list of version managers.

    Args:
        func:

    Returns:

    """
    request = get_request_from_args(*args, **kwargs)
    # super user
    if request and request.user.is_superuser:
        return func(*args, **kwargs)

    document_list = func(*args, **kwargs)
    for template in document_list:
        # anonymous user
        if not request or request.user.is_anonymous:
            if template.user:
                raise AccessControlError(
                    "Template: The user doesn't have enough rights."
                )
            if not settings.CAN_ANONYMOUS_ACCESS_PUBLIC_DOCUMENT:
                raise AccessControlError(
                    "Template: The user doesn't have enough rights."
                )

        # user is set
        if template.user and template.user != str(request.user.id):
            raise AccessControlError(
                "Template: The user doesn't have enough rights."
            )

    return document_list


def check_can_write(template, *args, **kwargs):
    """Check can write template.

    Args:
        template:
        args:
        kwargs:

    Returns:

    """
    request = get_request_from_args(*args, **kwargs)

    # super user
    if request.user.is_superuser:
        return True

    # anonymous
    if request.user.is_anonymous:
        raise AccessControlError(
            "Template: The user doesn't have enough rights."
        )

    # user is set
    if template.user:
        if template.user == str(request.user.id):
            return True
    # user is not set
    else:
        if request.user.is_staff:
            return True

    raise AccessControlError("Template: The user doesn't have enough rights.")



""" REST views for the template version manager API
"""
from django.http import Http404
from django.utils.decorators import method_decorator
from rest_framework import status
from rest_framework.exceptions import ValidationError
from rest_framework.permissions import IsAdminUser, IsAuthenticated
from rest_framework.response import Response
from rest_framework.views import APIView
from core_main_app.commons import exceptions as exceptions
from core_main_app.components.template_version_manager import (
    api as template_version_manager_api,
)
from core_main_app.components.version_manager import api as version_manager_api
from core_main_app.rest.template_version_manager.abstract_views import (
    AbstractTemplateVersionManagerList,
    AbstractTemplateList,
    AbstractStatusTemplateVersion,
    AbstractStatusTemplateVersionManager,
    AbstractTemplateVersionManagerDetail,
    AbstractOrderingTemplateVersionManager,
)
from core_main_app.rest.template_version_manager.serializers import (
    TemplateVersionManagerSerializer,
    CreateTemplateSerializer,
)
from core_main_app.utils.decorators import api_staff_member_required


class GlobalTemplateVersionManagerList(AbstractTemplateVersionManagerList):
    """List all GlobalTemplateVersionManager"""

    def get_template_version_managers(self):
        """Get GlobalTemplateVersionManager

        Returns:

            List of GlobalTemplateVersionManager
        """
        return template_version_manager_api.get_global_version_managers(
            request=self.request
        )


class UserTemplateVersionManagerList(AbstractTemplateVersionManagerList):
    """List all UserTemplateVersionManager"""

    permission_classes = (IsAuthenticated,)

    def get_template_version_managers(self):
        """Get all UserTemplateVersionManager

        Returns:

            List of UserTemplateVersionManager
        """
        return template_version_manager_api.get_all_by_user_id(
            request=self.request
        )


class GlobalAndUserTemplateVersionManagerList(
    AbstractTemplateVersionManagerList
):
    """List Global And User Template Version Manager"""

    permission_classes = (IsAdminUser,)

    def get_template_version_managers(self):
        """Get all Template Version Manager

        Returns:

            List of Template Version Manager
        """

        return template_version_manager_api.get_all(request=self.request)


class TemplateVersionManagerDetail(APIView):
    """Retrieve a TemplateVersionManager"""

    permission_classes = (IsAuthenticated,)

    def get_object(self, pk, request):
        """Get TemplateVersionManager from db

        Args:

            pk: ObjectId
            request:

        Returns:

            TemplateVersionManager
        """
        try:
            return template_version_manager_api.get_by_id(pk, request=request)
        except exceptions.DoesNotExist:
            raise Http404

    def get(self, request, pk):
        """Retrieve a TemplateVersionManager

        Args:

            request: HTTP request
            pk: ObjectId

        Returns:

            - code: 200
              content: TemplateVersionManager
            - code: 404
              content: Object was not found
            - code: 500
              content: Internal server error
        """
        try:
            # Get object
            template_version_manager_object = self.get_object(
                pk, request=request
            )

            # Serialize object
            serializer = TemplateVersionManagerSerializer(
                template_version_manager_object
            )

            # Return response
            return Response(serializer.data)
        except Http404:
            content = {"message": "Template Version Manager not found."}
            return Response(content, status=status.HTTP_404_NOT_FOUND)
        except Exception as api_exception:
            content = {"message": str(api_exception)}
            return Response(
                content, status=status.HTTP_500_INTERNAL_SERVER_ERROR
            )


class TemplateVersion(AbstractTemplateVersionManagerDetail):
    """Create a TemplateVersion"""

    @method_decorator(api_staff_member_required())
    def post(self, request, pk):
        """Create a TemplateVersion

        Parameters:

            {
                "filename": "filename",
                "content": "<xs:schema xmlns:xs='http://www.w3.org/2001/XMLSchema'><xs:element name='root'/></xs:schema>"
            }

        Note:

            "dependencies_dict": json.dumps({"schemaLocation1": "id1" ,"schemaLocation2":"id2"})

        Args:

            request: HTTP request

        Returns:

            - code: 200
              content: Created TemplateVersionManager
            - code: 400
              content: Validation error
            - code: 404
              content: Template was not found
            - code: 500
              content: Internal server error
        """
        try:
            # Get object
            template_version_manager_object = self.get_object(pk)

            # Build serializers
            template_serializer = CreateTemplateSerializer(
                data=request.data, context={"request": request}
            )

            # Validate data
            template_serializer.is_valid(raise_exception=True)

            # Save data
            template_serializer.save(
                template_version_manager=template_version_manager_object,
                user=template_version_manager_object.user,
            )

            return Response(
                template_serializer.data, status=status.HTTP_201_CREATED
            )
        except Http404:
            content = {"message": "Template Version Manager not found."}
            return Response(content, status=status.HTTP_404_NOT_FOUND)
        except ValidationError as validation_exception:
            content = {"message": validation_exception.detail}
            return Response(content, status=status.HTTP_400_BAD_REQUEST)
        except Exception as api_exception:
            content = {"message": str(api_exception)}
            return Response(
                content, status=status.HTTP_500_INTERNAL_SERVER_ERROR
            )


class UserTemplateList(AbstractTemplateList):
    """Create a Template (linked to the user)"""

    permission_classes = (IsAuthenticated,)

    def post(self, request):
        """Create a Template (linked to the user)

        Parameters:

            {
                "title": "title",
                "filename": "filename",
                "content": "<xs:schema xmlns:xs='http://www.w3.org/2001/XMLSchema'><xs:element name='root'/></xs:schema>"
            }

        Note:

            "dependencies_dict": json.dumps({"schemaLocation1": "id1" ,"schemaLocation2":"id2"})

        Args:

            request: HTTP request

        Returns:

            - code: 201
              content: Created Template
            - code: 400
              content: Validation error / not unique / XSD error
            - code: 500
              content: Internal server error
        """
        return super().post(request)

    def get_user(self):
        """Retrieve the user from the request

        Returns:

            User ID
        """
        return str(self.request.user.id)


class GlobalTemplateList(AbstractTemplateList):
    """Create a Template (global schema)"""

    @method_decorator(api_staff_member_required())
    def post(self, request):
        """Create a Template (global schema)

        Parameters:

            {
                "title": "title",
                "filename": "filename",
                "content": "<xs:schema xmlns:xs='http://www.w3.org/2001/XMLSchema'><xs:element name='root'/></xs:schema>"
            }

        Note:

            "dependencies_dict": json.dumps({"schemaLocation1": "id1" ,"schemaLocation2":"id2"})

        Args:

            request: HTTP request

        Returns:

            - code: 201
              content: Created template
            - code: 400
              content: Validation error / not unique / XSD error
            - code: 500
              content: Internal server error
        """
        return super().post(request)

    def get_user(self):
        """The user is None for a global template

        Returns:

            None
        """
        return None


class CurrentTemplateVersion(AbstractStatusTemplateVersion):
    """Update status to current"""

    permission_classes = (IsAuthenticated,)

    def status_update(self, template_object):
        """Update status to current

        Args:

            template_object: template_version

        Returns:

            TemplateVersion
        """
        return version_manager_api.set_current(
            template_object, request=self.request
        )


class DisableTemplateVersion(AbstractStatusTemplateVersion):
    """Update status to disabled"""

    permission_classes = (IsAuthenticated,)

    def status_update(self, template_object):
        """Update status to disabled

        Args:

            template_object: template_version

        Returns:

            TemplateVersion
        """
        return version_manager_api.disable_version(
            template_object, request=self.request
        )


class RestoreTemplateVersion(AbstractStatusTemplateVersion):
    """Update status to restored"""

    permission_classes = (IsAuthenticated,)

    def status_update(self, template_object):
        """Update status to restored

        Args:

            template_object: template_version

        Returns:

            TemplateVersion
        """
        return version_manager_api.restore_version(
            template_object, request=self.request
        )


class DisableTemplateVersionManager(AbstractStatusTemplateVersionManager):
    """Update status to disabled"""

    permission_classes = (IsAuthenticated,)

    def status_update(self, template_version_manager_object):
        """Update status to disabled

        Args:

            template_version_manager_object: template_version_manager

        Returns:

            TemplateVersionManager
        """
        # FIXME: add return?
        version_manager_api.disable(
            template_version_manager_object, request=self.request
        )


class RestoreTemplateVersionManager(AbstractStatusTemplateVersionManager):
    """Update status to restored"""

    permission_classes = (IsAuthenticated,)

    def status_update(self, template_version_manager_object):
        """Update status to restored

        Args:

            template_version_manager_object: template_version_manager

        """
        # FIXME: add return?
        version_manager_api.restore(
            template_version_manager_object, request=self.request
        )


class GlobalTemplateVersionManagerOrdering(
    AbstractOrderingTemplateVersionManager
):
    permission_classes = (IsAdminUser,)
    """Get Global TemplateVersionManager ordering"""

    def get_objects(self):
        """Get TemplateVersionManager list from db

        Args:

        Returns:
            TemplateVersionManager list
        """
        return template_version_manager_api.get_global_version_managers(
            request=self.request
        )

    def update_ordering(self, list_ids):
        """update global TemplateVersionManager ordering

        Args:
            list_ids:
        Returns:

        """
        template_version_manager_api.update_global_template_ordering(
            list_ids, request=self.request
        )


class UserTemplateVersionManagerOrdering(
    AbstractOrderingTemplateVersionManager
):
    """Get User TemplateVersionManager ordering"""

    def get_objects(self):
        """Get TemplateVersionManager list from db

        Args:

        Returns:
            TemplateVersionManager list
        """
        return template_version_manager_api.get_all_by_user_id(
            request=self.request
        )

    def update_ordering(self, list_ids):
        """update user TemplateVersionManager ordering

        Args:
            list_ids:
        Returns:

        """
        template_version_manager_api.update_user_template_ordering(
            list_ids, request=self.request
        )



""" Urls utils
"""

import re

from django.urls import reverse, re_path, include
from django.conf import settings


def get_template_download_pattern():
    """Return regex pattern to match an url to download a template.

    Returns:

    """
    # build django url to download a template
    download_template_url = reverse(
        "core_main_app_rest_template_download", kwargs={"pk": "template_id"}
    )
    # make url a regex
    download_template_regex = download_template_url.replace(
        "template_id", r"(?P<pk>\w+)"
    )
    # compile regex
    pattern = re.compile(download_template_regex)

    return pattern


def get_blob_download_regex(xml_string):
    """Return regex pattern to match an url to download a blob.

    Returns:

    """
    # build django url to download a blob
    download_blob_url = reverse(
        "core_main_app_rest_blob_download", kwargs={"pk": "blob_id"}
    )
    download_blob_url = download_blob_url.replace("blob_id/", "")
    # make the regex
    regex = ">(http[s]?:[^<>]+" + download_blob_url + "[0-9]+/?)<"
    return re.findall(regex, xml_string)


def get_auth_urls():
    """Get auth urls

    Returns:

    """
    urlpatterns = []

    if settings.ENABLE_SAML2_SSO_AUTH:
        from djangosaml2 import views as saml2_views

        urlpatterns.append(re_path(r"saml2/", include("djangosaml2.urls")))
        urlpatterns.append(
            re_path(
                r"^saml2/login",
                saml2_views.LoginView.as_view(),
                name="core_main_app_login",
            )
        )
        urlpatterns.append(
            re_path(
                r"^saml2/logout",
                saml2_views.LogoutInitView.as_view(),
                name="core_main_app_logout",
            )
        )
    else:
        from core_main_app.views.user import views as user_views
        from django.contrib.auth import views as auth_views

        urlpatterns.append(
            re_path(
                r"^login",
                user_views.custom_login,
                name="core_main_app_login",
            )
        )
        urlpatterns.append(
            re_path(
                r"^logout",
                user_views.custom_logout,
                name="core_main_app_logout",
            )
        )
        urlpatterns.append(
            re_path(
                "password_change/$",
                auth_views.PasswordChangeView.as_view(),
                name="password_change",
            )
        )
        urlpatterns.append(
            re_path(
                "password_change/done/$",
                auth_views.PasswordChangeDoneView.as_view(),
                name="password_change_done",
            )
        )
        urlpatterns.append(
            re_path(
                "password_reset/$",
                auth_views.PasswordResetView.as_view(),
                name="password_reset",
            )
        )
        urlpatterns.append(
            re_path(
                "password_reset/done/$",
                auth_views.PasswordResetDoneView.as_view(),
                name="password_reset_done",
            )
        )
        urlpatterns.append(
            re_path(
                r"^reset/(?P<uidb64>[0-9A-Za-z_\-]+)/(?P<token>[0-9A-Za-z]{1,13}-[0-9A-Za-z]{1,20})/$",
                auth_views.PasswordResetConfirmView.as_view(),
                name="password_reset_confirm",
            )
        )
        urlpatterns.append(
            re_path(
                r"^reset/done/$",
                auth_views.PasswordResetCompleteView.as_view(),
                name="password_reset_complete",
            )
        )
    return urlpatterns



""" Admin Site test class
"""
from unittest import TestCase
from unittest.mock import patch

from django.contrib.admin import ModelAdmin
from django.test import override_settings
from simple_history.admin import SimpleHistoryAdmin

from core_main_app.utils.admin_site.model_admin_class import (
    get_base_model_admin_class,
    register_simple_history_models,
)


class TestGetBaseModelAdminClass(TestCase):
    """TestGetBaseModelAdminClass"""

    @override_settings(DJANGO_SIMPLE_HISTORY_MODELS=["Data"])
    def test_get_base_model_admin_class_model_in_list(self):
        """test_get_base_model_admin_class_model_in_list

        Returns:

        """
        base_class = get_base_model_admin_class("Data")
        self.assertEquals(base_class, SimpleHistoryAdmin)

    @override_settings(DJANGO_SIMPLE_HISTORY_MODELS=["Data"])
    def test_get_base_model_admin_class_model_not_in_list(self):
        """test_get_base_model_admin_class_model_not_in_list

        Returns:

        """
        base_class = get_base_model_admin_class("Template")
        self.assertEquals(base_class, ModelAdmin)

    @override_settings(DJANGO_SIMPLE_HISTORY_MODELS=None)
    def test_get_base_model_admin_class_model_setting_not_set(self):
        """test_get_base_model_admin_class_model_setting_not_set

        Returns:

        """
        base_class = get_base_model_admin_class("Data")
        self.assertEquals(base_class, ModelAdmin)


class TestRegisterSimpleHistoryModels(TestCase):
    """TestRegisterSimpleHistoryModels"""

    @patch("simple_history.register")
    def test_register_simple_history_models_with_available_model(
        self, mock_register
    ):
        """test_register_simple_history_models_with_available_model

        Returns:

        """
        mock_register.return_value = None
        register_simple_history_models(django_simple_history_models=["Data"])
        self.assertTrue(mock_register.called)

    @patch("simple_history.register")
    def test_register_simple_history_models_with_unavailable_model(
        self, mock_register
    ):
        """test_register_simple_history_models_with_unavailable_model

        Returns:

        """
        mock_register.return_value = None
        register_simple_history_models(
            django_simple_history_models=["Template"]
        )
        self.assertFalse(mock_register.called)



# Generated by Django 3.2 on 2022-05-25 13:50

from django.db import migrations, models
import django.db.models.expressions


class Migration(migrations.Migration):

    dependencies = [
        ("core_main_app", "0003_psql_full_text"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="templateversionmanager",
            options={
                "ordering": [
                    django.db.models.expressions.OrderBy(
                        django.db.models.expressions.F("display_rank"),
                        nulls_last=True,
                    )
                ]
            },
        ),
        migrations.AddField(
            model_name="templateversionmanager",
            name="display_rank",
            field=models.IntegerField(blank=True, null=True),
        ),
    ]



""" Unit tests for `core_main_app.access_control.api`.
"""
from unittest import TestCase
from unittest.mock import MagicMock, patch

from core_main_app.access_control import api as access_control_api
from core_main_app.utils.tests_tools.MockUser import create_mock_user


class TestCanAnonymousAccessPublicData(TestCase):
    """Unit tests for `can_anonymous_access_public_data` function."""

    @patch.object(access_control_api, "check_anonymous_access")
    def test_check_anonymous_access_called(self, mock_check_anonymous_access):
        """test_check_anonymous_access_called"""
        user = create_mock_user("1")

        access_control_api.can_anonymous_access_public_data(
            MagicMock(), *[user]
        )

        mock_check_anonymous_access.assert_called_with(user)

    @patch.object(access_control_api, "check_anonymous_access")
    def test_returns_function(self, mock_check_anonymous_access):
        """test_returns_function"""
        func = MagicMock()
        user = create_mock_user("1")

        access_control_api.can_anonymous_access_public_data(func, *[user])

        func.assert_called()



""" Mongoengine paginator
"""
import inspect
import logging

from django.core.paginator import Paginator as DjangoPaginator
from django.utils.functional import cached_property

logger = logging.getLogger(__name__)


class MongoenginePaginator(DjangoPaginator):
    """Mongoengine Paginator"""

    @cached_property
    def count(self):
        """Return the total number of objects, across all pages. Override for mongoengine compatibility.

        Explanation:
            count method in /django/core/paginator.py checks method_has_no_args(c)
            where c is queryset.count() method.

            count method in /mongoengine/queryset/queryset.py has a with_limit_and_skip arg, thus the test fails and
            this results in len(queryset) being called (bad for performances)
        """
        count_func = getattr(self.object_list, "count", None)
        if callable(count_func) and not inspect.isbuiltin(count_func):
            return count_func()

        logger.warning(
            "count() function was not called. Calling len() instead."
        )
        return len(self.object_list)



""" Core main app user views
"""
from django.conf import settings
from django.contrib import auth as django_auth
from django.contrib.auth.decorators import login_required
from django.contrib.staticfiles import finders
from django.http.response import HttpResponse
from django.shortcuts import redirect
from django.urls import reverse
from django.utils import timezone
from pytz import common_timezones as pytz_common_timezones
from rest_framework.status import HTTP_405_METHOD_NOT_ALLOWED

from core_main_app.access_control.exceptions import AccessControlError
from core_main_app.commons.exceptions import DoesNotExist
from core_main_app.components.template_version_manager import (
    api as template_version_manager_api,
)
from core_main_app.components.user_preferences import (
    api as user_preferences_api,
)
from core_main_app.components.user_preferences.models import UserPreferences
from core_main_app.components.web_page_login import api as web_page_login_api
from core_main_app.utils.markdown_parser import parse
from core_main_app.utils.rendering import render
from core_main_app.views.user.forms import LoginForm

if "defender" in settings.INSTALLED_APPS:
    from defender.decorators import watch_login

    @watch_login()
    def defender_custom_login(request):
        """Custom login page with defender controls.

        Args:
            request:

        Returns:

        """
        return default_custom_login(request)


def custom_login(request):
    """Custom login page calls default page or page with defender if installed.

    Args:
        request:

    Returns:

    """
    if "defender" in settings.INSTALLED_APPS:
        return defender_custom_login(request)
    else:
        return default_custom_login(request)


def default_custom_login(request):
    """Default custom login page.

    Parameters:
        request:

    Returns:
    """

    def _login_redirect(to_page):
        if to_page is not None and to_page != "":
            return redirect(to_page)

        return redirect(reverse("core_main_app_homepage"))

    context = {
        "page_title": "Login",
        "with_website_features": "core_website_app" in settings.INSTALLED_APPS,
        "ENABLE_SAML2_SSO_AUTH": settings.ENABLE_SAML2_SSO_AUTH,
    }
    if request.method == "POST":
        username = request.POST["username"]
        password = request.POST["password"]
        next_page = request.POST["next_page"]

        try:
            user = django_auth.authenticate(
                username=username, password=password
            )
            if not user.is_active:
                context.update(
                    {
                        "login_form": LoginForm(
                            initial={"next_page": next_page}
                        ),
                        "login_locked": True,
                    }
                )
                return render(
                    request,
                    "core_main_app/user/login/main.html",
                    context=context,
                )

            django_auth.login(request, user)

            return _login_redirect(next_page)
        except Exception:
            context.update(
                {
                    "login_form": LoginForm(initial={"next_page": next_page}),
                    "login_error": True,
                }
            )
            return render(
                request,
                "core_main_app/user/login/main.html",
                context=context,
            )
    elif request.method == "GET":
        if request.user.is_authenticated:
            return redirect(reverse("core_main_app_homepage"))

        next_page = None
        if "next" in request.GET:
            next_page = request.GET["next"]

        # build the context
        context.update(
            {"login_form": LoginForm(initial={"next_page": next_page})}
        )
        assets = {"css": ["core_main_app/user/css/login.css"]}

        # get the web page login if exist
        web_page_login = web_page_login_api.get()

        if web_page_login:
            # update the context
            context["login_message"] = parse(web_page_login.content)

            # if exist we build assets and modals collection
            assets.update(
                {
                    "js": [
                        {
                            "path": "core_main_app/user/js/login/message.js",
                            "is_raw": False,
                        }
                    ]
                }
            )
            modals = ["core_main_app/user/login/modals/login_message.html"]

            # render the page with context, assets and modals
            return render(
                request,
                "core_main_app/user/login/main.html",
                context=context,
                assets=assets,
                modals=modals,
            )

        # render the page with context, assets
        return render(
            request,
            "core_main_app/user/login/main.html",
            context=context,
            assets=assets,
        )
    else:
        return HttpResponse(status=HTTP_405_METHOD_NOT_ALLOWED)


def custom_logout(request):
    """Custom logout page.

    Parameters:
        request:

    Returns:
    """
    django_auth.logout(request)
    return redirect(reverse("core_main_app_homepage"))


def homepage(request):
    """Homepage for the website

    Parameters:
        request:

    Returns:
    """
    assets = dict()

    if finders.find("core_main_app/css/homepage.css") is not None:
        assets["css"] = ["core_main_app/css/homepage.css"]

    if finders.find("core_main_app/js/homepage.js") is not None:
        assets["js"] = [
            {"path": "core_main_app/js/homepage.js", "is_raw": False}
        ]

    return render(request, "core_main_app/user/homepage.html", assets=assets)


@login_required
def manage_template_versions(request, version_manager_id):
    """View that allows template versions management

    Args:
        request:
        version_manager_id:

    Returns:

    """
    try:
        # get the version manager
        version_manager = template_version_manager_api.get_by_id(
            version_manager_id, request=request
        )

        if not request.user.is_staff and version_manager.user != str(
            request.user.id
        ):
            raise Exception(
                "You do not have the rights to perform this action."
            )

        context = get_context_manage_template_versions(version_manager)
        if "core_parser_app" in settings.INSTALLED_APPS:
            context.update({"module_url": "core_parser_app_template_modules"})

        assets = {
            "js": [
                {
                    "path": "core_main_app/common/js/templates/versions/set_current.js",
                    "is_raw": False,
                },
                {
                    "path": "core_main_app/common/js/templates/versions/restore.js",
                    "is_raw": False,
                },
                {
                    "path": "core_main_app/common/js/templates/versions/modals/disable.js",
                    "is_raw": False,
                },
            ]
        }

        modals = ["core_main_app/admin/templates/versions/modals/disable.html"]

        # Set page title
        context.update({"page_title": "Template Versions"})

        return render(
            request,
            "core_main_app/common/templates/versions.html",
            assets=assets,
            modals=modals,
            context=context,
        )
    except Exception as e:
        return render(
            request,
            "core_main_app/common/commons/error.html",
            context={"error": str(e), "page_title": "Error"},
        )


def get_context_manage_template_versions(
    version_manager, object_name="Template"
):
    """Get the context to manage the template versions.

    Args:
        version_manager:
        object_name:

    Returns:

    """

    # Use categorized version for easier manipulation in template
    versions = version_manager.version_set
    categorized_versions = {"available": [], "disabled": []}
    for index, version in enumerate(versions, 1):
        indexed_version = {
            "index": index,
            "object": str(version.id),
            "creation_date": version.creation_date,
        }

        if str(version.id) not in version_manager.disabled_versions:
            categorized_versions["available"].append(indexed_version)
        else:
            categorized_versions["disabled"].append(indexed_version)

    context = {
        "object_name": object_name,
        "version_manager": version_manager,
        "categorized_versions": categorized_versions,
    }

    return context


def saml2_failure(request, exception=None, status=403, **kwargs):
    """Renders a simple template with an error message."""

    return render(
        request,
        "core_main_app/user/login/saml2_error.html",
        context={
            "with_website_features": "core_website_app"
            in settings.INSTALLED_APPS,
            "page_title": "Error",
        },
    )


@login_required
def set_timezone(request):
    """Set timezone in session

    Args:
        request:

    Returns:

    """
    try:
        if request.method == "POST":
            user_preferences = _get_preferences(request.user)
            if not user_preferences:
                user_preferences = UserPreferences(
                    user_id=str(request.user.id)
                )
            user_preferences.timezone = request.POST["timezone"]
            user_preferences_api.upsert(user_preferences, request.user)
            request.session["django_timezone"] = user_preferences.timezone
            return redirect("/")
        else:
            user_preferences = _get_preferences(request.user)
            user_timezone = (
                user_preferences.timezone
                if user_preferences and user_preferences.timezone
                else timezone.get_current_timezone().key
            )
            return render(
                request,
                "core_main_app/user/timezone.html",
                context={
                    "timezones": pytz_common_timezones,
                    "timezone": user_timezone,
                },
            )
    except AccessControlError:
        error_message = "Access Forbidden"
        status_code = 403
    except Exception as exception:
        error_message = f"An error occurred: {str(exception)}"
        status_code = 400

    return render(
        request,
        "core_main_app/common/commons/error.html",
        context={
            "error": error_message,
            "status_code": status_code,
            "page_title": "Error",
        },
    )


def _get_preferences(user):
    """Get User Preferences

    Args:
        user: User

    Return:
    """
    try:
        return user_preferences_api.get_by_user(user)
    except DoesNotExist:
        return None



""" Fixtures files for Group
"""
from core_main_app.components.group import api as group_api
from core_main_app.utils.integration_tests.fixture_interface import (
    FixtureInterface,
)


class GroupFixtures(FixtureInterface):
    """Group Fixture"""

    def insert_data(self):
        """insert data

        Returns:

        """
        pass

    @staticmethod
    def create_group(name="name"):
        """create group

        Args:
            name:

        Returns:

        """
        group, created = group_api.get_or_create(name=name)
        return group



"""Parser util for text editor
"""


def get_parser(request=None):
    """Load configuration for the parser.

    Args:
        request:

    Returns:

    """
    from core_parser_app.tools.parser.parser import XSDParser

    return XSDParser(
        min_tree=True,
        ignore_modules=True,
        collapse=True,
        auto_key_keyref=False,
        implicit_extension_base=False,
        download_dependencies=False,
        request=request,
    )
